@@ -75,28 +75,30 @@ static PyObject* _cbson_query_message(PyObject* self, PyObject* args) {
     int num_to_return;
     PyObject* query;
     PyObject* field_selector;
+    PyObject* options_obj;
     codec_options_t options;
     buffer_t buffer = NULL;
     int length_location, message_length;
     PyObject* result = NULL;
 
-    if (!PyArg_ParseTuple(args, "Iet#iiOOO&",
+    if (!(PyArg_ParseTuple(args, "Iet#iiOOO",
                           &flags,
                           "utf-8",
                           &collection_name,
                           &collection_name_length,
                           &num_to_skip, &num_to_return,
                           &query, &field_selector,
-                          convert_codec_options, &options)) {
+                          &options_obj) &&
+            convert_codec_options(state->_cbson, options_obj, &options))) {
         return NULL;
     }
-    buffer = buffer_new();
+    buffer = pymongo_buffer_new();
     if (!buffer) {
         goto fail;
     }
 
     // save space for message length
-    length_location = buffer_save_space(buffer, 4);
+    length_location = pymongo_buffer_save_space(buffer, 4);
     if (length_location == -1) {
         goto fail;
     }
@@ -111,37 +113,37 @@ static PyObject* _cbson_query_message(PyObject* self, PyObject* args) {
         goto fail;
     }
 
-    begin = buffer_get_position(buffer);
+    begin = pymongo_buffer_get_position(buffer);
     if (!write_dict(state->_cbson, buffer, query, 0, &options, 1)) {
         goto fail;
     }
 
-    max_size = buffer_get_position(buffer) - begin;
+    max_size = pymongo_buffer_get_position(buffer) - begin;
 
     if (field_selector != Py_None) {
-        begin = buffer_get_position(buffer);
+        begin = pymongo_buffer_get_position(buffer);
         if (!write_dict(state->_cbson, buffer, field_selector, 0,
                         &options, 1)) {
             goto fail;
         }
-        cur_size = buffer_get_position(buffer) - begin;
+        cur_size = pymongo_buffer_get_position(buffer) - begin;
         max_size = (cur_size > max_size) ? cur_size : max_size;
     }
 
-    message_length = buffer_get_position(buffer) - length_location;
+    message_length = pymongo_buffer_get_position(buffer) - length_location;
     buffer_write_int32_at_position(
         buffer, length_location, (int32_t)message_length);
 
     /* objectify buffer */
     result = Py_BuildValue("iy#i", request_id,
-                           buffer_get_buffer(buffer),
-                           (Py_ssize_t)buffer_get_position(buffer),
+                           pymongo_buffer_get_buffer(buffer),
+                           (Py_ssize_t)pymongo_buffer_get_position(buffer),
                            max_size);
 fail:
     PyMem_Free(collection_name);
     destroy_codec_options(&options);
     if (buffer) {
-        buffer_free(buffer);
+        pymongo_buffer_free(buffer);
     }
     return result;
 }
@@ -165,13 +167,13 @@ static PyObject* _cbson_get_more_message(PyObject* self, PyObject* args) {
                           &cursor_id)) {
         return NULL;
     }
-    buffer = buffer_new();
+    buffer = pymongo_buffer_new();
     if (!buffer) {
         goto fail;
     }
 
     // save space for message length
-    length_location = buffer_save_space(buffer, 4);
+    length_location = pymongo_buffer_save_space(buffer, 4);
     if (length_location == -1) {
         goto fail;
     }
@@ -188,18 +190,18 @@ static PyObject* _cbson_get_more_message(PyObject* self, PyObject* args) {
         goto fail;
     }
 
-    message_length = buffer_get_position(buffer) - length_location;
+    message_length = pymongo_buffer_get_position(buffer) - length_location;
     buffer_write_int32_at_position(
         buffer, length_location, (int32_t)message_length);
 
     /* objectify buffer */
     result = Py_BuildValue("iy#", request_id,
-                           buffer_get_buffer(buffer),
-                           (Py_ssize_t)buffer_get_position(buffer));
+                           pymongo_buffer_get_buffer(buffer),
+                           (Py_ssize_t)pymongo_buffer_get_position(buffer));
 fail:
     PyMem_Free(collection_name);
     if (buffer) {
-        buffer_free(buffer);
+        pymongo_buffer_free(buffer);
     }
     return result;
 }
@@ -220,6 +222,7 @@ static PyObject* _cbson_op_msg(PyObject* self, PyObject* args) {
     Py_ssize_t identifier_length = 0;
     PyObject* docs;
     PyObject* doc;
+    PyObject* options_obj;
     codec_options_t options;
     buffer_t buffer = NULL;
     int length_location, message_length;
@@ -229,23 +232,24 @@ static PyObject* _cbson_op_msg(PyObject* self, PyObject* args) {
     PyObject* iterator = NULL;
 
     /*flags, command, identifier, docs, opts*/
-    if (!PyArg_ParseTuple(args, "IOet#OO&",
+    if (!(PyArg_ParseTuple(args, "IOet#OO",
                           &flags,
                           &command,
                           "utf-8",
                           &identifier,
                           &identifier_length,
                           &docs,
-                          convert_codec_options, &options)) {
+                          &options_obj) &&
+            convert_codec_options(state->_cbson, options_obj, &options))) {
         return NULL;
     }
-    buffer = buffer_new();
+    buffer = pymongo_buffer_new();
     if (!buffer) {
         goto fail;
     }
 
     // save space for message length
-    length_location = buffer_save_space(buffer, 4);
+    length_location = pymongo_buffer_save_space(buffer, 4);
     if (length_location == -1) {
         goto fail;
     }
@@ -273,7 +277,7 @@ static PyObject* _cbson_op_msg(PyObject* self, PyObject* args) {
             goto fail;
         }
         /* save space for payload 0 length */
-        payload_one_length_location = buffer_save_space(buffer, 4);
+        payload_one_length_location = pymongo_buffer_save_space(buffer, 4);
         /* C string identifier */
         if (!buffer_write_bytes_ssize_t(buffer, identifier, identifier_length + 1)) {
             goto fail;
@@ -295,26 +299,26 @@ static PyObject* _cbson_op_msg(PyObject* self, PyObject* args) {
             Py_CLEAR(doc);
         }
 
-        payload_length = buffer_get_position(buffer) - payload_one_length_location;
+        payload_length = pymongo_buffer_get_position(buffer) - payload_one_length_location;
         buffer_write_int32_at_position(
             buffer, payload_one_length_location, (int32_t)payload_length);
         total_size += payload_length;
     }
 
-    message_length = buffer_get_position(buffer) - length_location;
+    message_length = pymongo_buffer_get_position(buffer) - length_location;
     buffer_write_int32_at_position(
         buffer, length_location, (int32_t)message_length);
 
     /* objectify buffer */
     result = Py_BuildValue("iy#ii", request_id,
-                           buffer_get_buffer(buffer),
-                           (Py_ssize_t)buffer_get_position(buffer),
+                           pymongo_buffer_get_buffer(buffer),
+                           (Py_ssize_t)pymongo_buffer_get_position(buffer),
                            total_size,
                            max_doc_size);
 fail:
     Py_XDECREF(iterator);
     if (buffer) {
-        buffer_free(buffer);
+        pymongo_buffer_free(buffer);
     }
     PyMem_Free(identifier);
     destroy_codec_options(&options);
@@ -400,7 +404,7 @@ _batched_op_msg(
         return 0;
     }
     /* Save space for size */
-    size_location = buffer_save_space(buffer, 4);
+    size_location = pymongo_buffer_save_space(buffer, 4);
     if (size_location == -1) {
         return 0;
     }
@@ -445,17 +449,17 @@ _batched_op_msg(
         return 0;
     }
     while ((doc = PyIter_Next(iterator)) != NULL) {
-        int cur_doc_begin = buffer_get_position(buffer);
+        int cur_doc_begin = pymongo_buffer_get_position(buffer);
         int cur_size;
         int doc_too_large = 0;
         int unacked_doc_too_large = 0;
         if (!write_dict(state->_cbson, buffer, doc, 0, &options, 1)) {
             goto fail;
         }
-        cur_size = buffer_get_position(buffer) - cur_doc_begin;
+        cur_size = pymongo_buffer_get_position(buffer) - cur_doc_begin;
 
         /* Does the first document exceed max_message_size? */
-        doc_too_large = (idx == 0 && (buffer_get_position(buffer) > max_message_size));
+        doc_too_large = (idx == 0 && (pymongo_buffer_get_position(buffer) > max_message_size));
         /* When OP_MSG is used unacknowledged we have to check
          * document size client side or applications won't be notified.
          * Otherwise we let the server deal with documents that are too large
@@ -483,12 +487,12 @@ _batched_op_msg(
             goto fail;
         }
         /* We have enough data, return this batch. */
-        if (buffer_get_position(buffer) > max_message_size) {
+        if (pymongo_buffer_get_position(buffer) > max_message_size) {
             /*
              * Roll the existing buffer back to the beginning
              * of the last document encoded.
              */
-            buffer_update_position(buffer, cur_doc_begin);
+            pymongo_buffer_update_position(buffer, cur_doc_begin);
             Py_CLEAR(doc);
             break;
         }
@@ -508,7 +512,7 @@ _batched_op_msg(
         goto fail;
     }
 
-    position = buffer_get_position(buffer);
+    position = pymongo_buffer_get_position(buffer);
     length = position - size_location;
     buffer_write_int32_at_position(buffer, size_location, (int32_t)length);
     return 1;
@@ -528,17 +532,18 @@ _cbson_encode_batched_op_msg(PyObject* self, PyObject* args) {
     PyObject* ctx = NULL;
     PyObject* to_publish = NULL;
     PyObject* result = NULL;
+    PyObject* options_obj;
     codec_options_t options;
     buffer_t buffer;
     struct module_state *state = GETSTATE(self);
 
-    if (!PyArg_ParseTuple(args, "bOObO&O",
+    if (!(PyArg_ParseTuple(args, "bOObOO",
                           &op, &command, &docs, &ack,
-                          convert_codec_options, &options,
-                          &ctx)) {
+                          &options_obj, &ctx) &&
+            convert_codec_options(state->_cbson, options_obj, &options))) {
         return NULL;
     }
-    if (!(buffer = buffer_new())) {
+    if (!(buffer = pymongo_buffer_new())) {
         destroy_codec_options(&options);
         return NULL;
     }
@@ -560,12 +565,12 @@ _cbson_encode_batched_op_msg(PyObject* self, PyObject* args) {
     }
 
     result = Py_BuildValue("y#O",
-                           buffer_get_buffer(buffer),
-                           (Py_ssize_t)buffer_get_position(buffer),
+                           pymongo_buffer_get_buffer(buffer),
+                           (Py_ssize_t)pymongo_buffer_get_position(buffer),
                            to_publish);
 fail:
     destroy_codec_options(&options);
-    buffer_free(buffer);
+    pymongo_buffer_free(buffer);
     Py_XDECREF(to_publish);
     return result;
 }
@@ -581,22 +586,23 @@ _cbson_batched_op_msg(PyObject* self, PyObject* args) {
     PyObject* ctx = NULL;
     PyObject* to_publish = NULL;
     PyObject* result = NULL;
+    PyObject* options_obj;
     codec_options_t options;
     buffer_t buffer;
     struct module_state *state = GETSTATE(self);
 
-    if (!PyArg_ParseTuple(args, "bOObO&O",
+    if (!(PyArg_ParseTuple(args, "bOObOO",
                           &op, &command, &docs, &ack,
-                          convert_codec_options, &options,
-                          &ctx)) {
+                          &options_obj, &ctx) &&
+            convert_codec_options(state->_cbson, options_obj, &options))) {
         return NULL;
     }
-    if (!(buffer = buffer_new())) {
+    if (!(buffer = pymongo_buffer_new())) {
         destroy_codec_options(&options);
         return NULL;
     }
     /* Save space for message length and request id */
-    if ((buffer_save_space(buffer, 8)) == -1) {
+    if ((pymongo_buffer_save_space(buffer, 8)) == -1) {
         goto fail;
     }
     if (!buffer_write_bytes(buffer,
@@ -623,16 +629,16 @@ _cbson_batched_op_msg(PyObject* self, PyObject* args) {
     }
 
     request_id = rand();
-    position = buffer_get_position(buffer);
+    position = pymongo_buffer_get_position(buffer);
     buffer_write_int32_at_position(buffer, 0, (int32_t)position);
     buffer_write_int32_at_position(buffer, 4, (int32_t)request_id);
     result = Py_BuildValue("iy#O", request_id,
-                           buffer_get_buffer(buffer),
-                           (Py_ssize_t)buffer_get_position(buffer),
+                           pymongo_buffer_get_buffer(buffer),
+                           (Py_ssize_t)pymongo_buffer_get_position(buffer),
                            to_publish);
 fail:
     destroy_codec_options(&options);
-    buffer_free(buffer);
+    pymongo_buffer_free(buffer);
     Py_XDECREF(to_publish);
     return result;
 }
@@ -702,14 +708,14 @@ _batched_write_command(
     }
 
     /* Position of command document length */
-    cmd_len_loc = buffer_get_position(buffer);
+    cmd_len_loc = pymongo_buffer_get_position(buffer);
     if (!write_dict(state->_cbson, buffer, command, 0,
                     &options, 0)) {
         return 0;
     }
 
     /* Write type byte for array */
-    *(buffer_get_buffer(buffer) + (buffer_get_position(buffer) - 1)) = 0x4;
+    *(pymongo_buffer_get_buffer(buffer) + (pymongo_buffer_get_position(buffer) - 1)) = 0x4;
 
     switch (op) {
     case _INSERT:
@@ -742,7 +748,7 @@ _batched_write_command(
     }
 
     /* Save space for list document */
-    lst_len_loc = buffer_save_space(buffer, 4);
+    lst_len_loc = pymongo_buffer_save_space(buffer, 4);
     if (lst_len_loc == -1) {
         return 0;
     }
@@ -757,17 +763,20 @@ _batched_write_command(
         return 0;
     }
     while ((doc = PyIter_Next(iterator)) != NULL) {
-        int sub_doc_begin = buffer_get_position(buffer);
+        int sub_doc_begin = pymongo_buffer_get_position(buffer);
         int cur_doc_begin;
         int cur_size;
         int enough_data = 0;
-        char key[16];
-        INT2STRING(key, idx);
+        char key[BUF_SIZE];
+        int res = LL2STR(key, (long long)idx);
+        if (res == -1) {
+            return 0;
+        }
         if (!buffer_write_bytes(buffer, "\x03", 1) ||
             !buffer_write_bytes(buffer, key, (int)strlen(key) + 1)) {
             goto fail;
         }
-        cur_doc_begin = buffer_get_position(buffer);
+        cur_doc_begin = pymongo_buffer_get_position(buffer);
         if (!write_dict(state->_cbson, buffer, doc, 0, &options, 1)) {
             goto fail;
         }
@@ -775,7 +784,7 @@ _batched_write_command(
         /* We have enough data, return this batch.
          * max_cmd_size accounts for the two trailing null bytes.
          */
-        cur_size = buffer_get_position(buffer) - cur_doc_begin;
+        cur_size = pymongo_buffer_get_position(buffer) - cur_doc_begin;
         /* This single document is too large for the command. */
         if (cur_size > max_cmd_size) {
             if (op == _INSERT) {
@@ -797,13 +806,13 @@ _batched_write_command(
             goto fail;
         }
         enough_data = (idx >= 1 &&
-                       (buffer_get_position(buffer) > max_split_size));
+                       (pymongo_buffer_get_position(buffer) > max_split_size));
         if (enough_data) {
             /*
              * Roll the existing buffer back to the beginning
              * of the last document encoded.
              */
-            buffer_update_position(buffer, sub_doc_begin);
+            pymongo_buffer_update_position(buffer, sub_doc_begin);
             Py_CLEAR(doc);
             break;
         }
@@ -827,7 +836,7 @@ _batched_write_command(
         goto fail;
     }
 
-    position = buffer_get_position(buffer);
+    position = pymongo_buffer_get_position(buffer);
     length = position - lst_len_loc - 1;
     buffer_write_int32_at_position(buffer, lst_len_loc, (int32_t)length);
     length = position - cmd_len_loc;
@@ -850,17 +859,18 @@ _cbson_encode_batched_write_command(PyObject* self, PyObject* args) {
     PyObject* ctx = NULL;
     PyObject* to_publish = NULL;
     PyObject* result = NULL;
+    PyObject* options_obj;
     codec_options_t options;
     buffer_t buffer;
     struct module_state *state = GETSTATE(self);
 
-    if (!PyArg_ParseTuple(args, "et#bOOO&O", "utf-8",
+    if (!(PyArg_ParseTuple(args, "et#bOOOO", "utf-8",
                           &ns, &ns_len, &op, &command, &docs,
-                          convert_codec_options, &options,
-                          &ctx)) {
+                          &options_obj, &ctx) &&
+            convert_codec_options(state->_cbson, options_obj, &options))) {
         return NULL;
     }
-    if (!(buffer = buffer_new())) {
+    if (!(buffer = pymongo_buffer_new())) {
         PyMem_Free(ns);
         destroy_codec_options(&options);
         return NULL;
@@ -884,13 +894,13 @@ _cbson_encode_batched_write_command(PyObject* self, PyObject* args) {
     }
 
     result = Py_BuildValue("y#O",
-                           buffer_get_buffer(buffer),
-                           (Py_ssize_t)buffer_get_position(buffer),
+                           pymongo_buffer_get_buffer(buffer),
+                           (Py_ssize_t)pymongo_buffer_get_position(buffer),
                            to_publish);
 fail:
     PyMem_Free(ns);
     destroy_codec_options(&options);
-    buffer_free(buffer);
+    pymongo_buffer_free(buffer);
     Py_XDECREF(to_publish);
     return result;
 }
