@@ -23,12 +23,11 @@ Causally Consistent Reads
 
   with client.start_session(causal_consistency=True) as session:
       collection = client.db.collection
-      collection.update_one({'_id': 1}, {'$set': {'x': 10}}, session=session)
-      secondary_c = collection.with_options(
-          read_preference=ReadPreference.SECONDARY)
+      collection.update_one({"_id": 1}, {"$set": {"x": 10}}, session=session)
+      secondary_c = collection.with_options(read_preference=ReadPreference.SECONDARY)
 
       # A secondary read waits for replication of the write.
-      secondary_c.find_one({'_id': 1}, session=session)
+      secondary_c.find_one({"_id": 1}, session=session)
 
 If `causal_consistency` is True (the default), read operations that use
 the session are causally after previous read and write operations. Using a
@@ -57,8 +56,11 @@ operation:
   with client.start_session() as session:
       with session.start_transaction():
           orders.insert_one({"sku": "abc123", "qty": 100}, session=session)
-          inventory.update_one({"sku": "abc123", "qty": {"$gte": 100}},
-                               {"$inc": {"qty": -100}}, session=session)
+          inventory.update_one(
+              {"sku": "abc123", "qty": {"$gte": 100}},
+              {"$inc": {"qty": -100}},
+              session=session,
+          )
 
 Upon normal completion of ``with session.start_transaction()`` block, the
 transaction automatically calls :meth:`ClientSession.commit_transaction`.
@@ -131,6 +133,8 @@ Classes
 =======
 """
 
+from __future__ import annotations
+
 import collections
 import time
 import uuid
@@ -140,7 +144,6 @@ from typing import (
     Any,
     Callable,
     ContextManager,
-    Generic,
     Mapping,
     NoReturn,
     Optional,
@@ -151,6 +154,7 @@ from bson.binary import Binary
 from bson.int64 import Int64
 from bson.son import SON
 from bson.timestamp import Timestamp
+from pymongo import _csot
 from pymongo.cursor import _SocketManager
 from pymongo.errors import (
     ConfigurationError,
@@ -164,11 +168,10 @@ from pymongo.helpers import _RETRYABLE_ERROR_CODES
 from pymongo.read_concern import ReadConcern
 from pymongo.read_preferences import ReadPreference, _ServerMode
 from pymongo.server_type import SERVER_TYPE
-from pymongo.typings import _DocumentType
 from pymongo.write_concern import WriteConcern
 
 
-class SessionOptions(object):
+class SessionOptions:
     """Options for a new :class:`ClientSession`.
 
     :Parameters:
@@ -202,8 +205,9 @@ class SessionOptions(object):
             if not isinstance(default_transaction_options, TransactionOptions):
                 raise TypeError(
                     "default_transaction_options must be an instance of "
-                    "pymongo.client_session.TransactionOptions, not: %r"
-                    % (default_transaction_options,)
+                    "pymongo.client_session.TransactionOptions, not: {!r}".format(
+                        default_transaction_options
+                    )
                 )
         self._default_transaction_options = default_transaction_options
         self._snapshot = snapshot
@@ -231,7 +235,7 @@ class SessionOptions(object):
         return self._snapshot
 
 
-class TransactionOptions(object):
+class TransactionOptions:
     """Options for :meth:`ClientSession.start_transaction`.
 
     :Parameters:
@@ -274,25 +278,25 @@ class TransactionOptions(object):
             if not isinstance(read_concern, ReadConcern):
                 raise TypeError(
                     "read_concern must be an instance of "
-                    "pymongo.read_concern.ReadConcern, not: %r" % (read_concern,)
+                    "pymongo.read_concern.ReadConcern, not: {!r}".format(read_concern)
                 )
         if write_concern is not None:
             if not isinstance(write_concern, WriteConcern):
                 raise TypeError(
                     "write_concern must be an instance of "
-                    "pymongo.write_concern.WriteConcern, not: %r" % (write_concern,)
+                    "pymongo.write_concern.WriteConcern, not: {!r}".format(write_concern)
                 )
             if not write_concern.acknowledged:
                 raise ConfigurationError(
                     "transactions do not support unacknowledged write concern"
-                    ": %r" % (write_concern,)
+                    ": {!r}".format(write_concern)
                 )
         if read_preference is not None:
             if not isinstance(read_preference, _ServerMode):
                 raise TypeError(
-                    "%r is not valid for read_preference. See "
+                    "{!r} is not valid for read_preference. See "
                     "pymongo.read_preferences for valid "
-                    "options." % (read_preference,)
+                    "options.".format(read_preference)
                 )
         if max_commit_time_ms is not None:
             if not isinstance(max_commit_time_ms, int):
@@ -339,12 +343,12 @@ def _validate_session_write_concern(session, write_concern):
             else:
                 raise ConfigurationError(
                     "Explicit sessions are incompatible with "
-                    "unacknowledged write concern: %r" % (write_concern,)
+                    "unacknowledged write concern: {!r}".format(write_concern)
                 )
     return session
 
 
-class _TransactionContext(object):
+class _TransactionContext:
     """Internal transaction context manager for start_transaction."""
 
     def __init__(self, session):
@@ -361,7 +365,7 @@ class _TransactionContext(object):
                 self.__session.abort_transaction()
 
 
-class _TxnState(object):
+class _TxnState:
     NONE = 1
     STARTING = 2
     IN_PROGRESS = 3
@@ -370,7 +374,7 @@ class _TxnState(object):
     ABORTED = 6
 
 
-class _Transaction(object):
+class _Transaction:
     """Internal class to hold transaction information in a ClientSession."""
 
     def __init__(self, opts, client):
@@ -436,7 +440,7 @@ def _max_time_expired_error(exc):
 
 # From the transactions spec, all the retryable writes errors plus
 # WriteConcernFailed.
-_UNKNOWN_COMMIT_ERROR_CODES = _RETRYABLE_ERROR_CODES | frozenset(
+_UNKNOWN_COMMIT_ERROR_CODES: frozenset = _RETRYABLE_ERROR_CODES | frozenset(
     [
         64,  # WriteConcernFailed
         50,  # MaxTimeMSExpired
@@ -461,7 +465,7 @@ if TYPE_CHECKING:
     from pymongo.mongo_client import MongoClient
 
 
-class ClientSession(Generic[_DocumentType]):
+class ClientSession:
     """A session for ordering sequential operations.
 
     :class:`ClientSession` instances are **not thread-safe or fork-safe**.
@@ -476,13 +480,13 @@ class ClientSession(Generic[_DocumentType]):
 
     def __init__(
         self,
-        client: "MongoClient[_DocumentType]",
+        client: MongoClient,
         server_session: Any,
         options: SessionOptions,
         implicit: bool,
     ) -> None:
         # A MongoClient, a _ServerSession, a SessionOptions, and a set.
-        self._client: MongoClient[_DocumentType] = client
+        self._client: MongoClient = client
         self._server_session = server_session
         self._options = options
         self._cluster_time = None
@@ -515,14 +519,14 @@ class ClientSession(Generic[_DocumentType]):
         if self._server_session is None:
             raise InvalidOperation("Cannot use ended session")
 
-    def __enter__(self) -> "ClientSession[_DocumentType]":
+    def __enter__(self) -> "ClientSession":
         return self
 
     def __exit__(self, exc_type: Any, exc_val: Any, exc_tb: Any) -> None:
         self._end_session(lock=True)
 
     @property
-    def client(self) -> "MongoClient[_DocumentType]":
+    def client(self) -> MongoClient:
         """The :class:`~pymongo.mongo_client.MongoClient` this session was
         created from.
         """
@@ -599,7 +603,7 @@ class ClientSession(Generic[_DocumentType]):
         In the event of an exception, ``with_transaction`` may retry the commit
         or the entire transaction, therefore ``callback`` may be invoked
         multiple times by a single call to ``with_transaction``. Developers
-        should be mindful of this possiblity when writing a ``callback`` that
+        should be mindful of this possibility when writing a ``callback`` that
         modifies application state or has any other side-effects.
         Note that even when the ``callback`` is invoked multiple times,
         ``with_transaction`` ensures that the transaction will be committed
@@ -828,7 +832,7 @@ class ClientSession(Generic[_DocumentType]):
         wc = opts.write_concern
         cmd = SON([(command_name, 1)])
         if command_name == "commitTransaction":
-            if opts.max_commit_time_ms:
+            if opts.max_commit_time_ms and _csot.get_timeout() is None:
                 cmd["maxTimeMS"] = opts.max_commit_time_ms
 
             # Transaction spec says that after the initial commit attempt,
@@ -972,7 +976,7 @@ class ClientSession(Generic[_DocumentType]):
             if read_preference != ReadPreference.PRIMARY:
                 raise InvalidOperation(
                     "read preference in a transaction must be primary, not: "
-                    "%r" % (read_preference,)
+                    "{!r}".format(read_preference)
                 )
 
             if self._transaction.state == _TxnState.STARTING:
@@ -1022,7 +1026,7 @@ class _EmptyServerSession:
         self.started_retryable_write = True
 
 
-class _ServerSession(object):
+class _ServerSession:
     def __init__(self, generation):
         # Ensure id is type 4, regardless of CodecOptions.uuid_representation.
         self.session_id = {"id": Binary(uuid.uuid4().bytes, 4)}
@@ -1061,7 +1065,7 @@ class _ServerSessionPool(collections.deque):
     """
 
     def __init__(self, *args, **kwargs):
-        super(_ServerSessionPool, self).__init__(*args, **kwargs)
+        super().__init__(*args, **kwargs)
         self.generation = 0
 
     def reset(self):
