@@ -17,6 +17,7 @@
 import sys
 import traceback
 from collections import abc
+from typing import Any, List, NoReturn
 
 from bson.son import SON
 from pymongo import ASCENDING
@@ -29,6 +30,7 @@ from pymongo.errors import (
     WriteConcernError,
     WriteError,
     WTimeoutError,
+    _wtimeout_error,
 )
 from pymongo.hello import HelloCompat
 
@@ -42,7 +44,7 @@ _SHUTDOWN_CODES = frozenset(
 # From the SDAM spec, the "not primary" error codes are combined with the
 # "node is recovering" error codes (of which the "node is shutting down"
 # errors are a subset).
-_NOT_PRIMARY_CODES = (
+_NOT_PRIMARY_CODES: frozenset = (
     frozenset(
         [
             10058,  # LegacyNotPrimary <=3.2 "not primary" error code
@@ -56,7 +58,7 @@ _NOT_PRIMARY_CODES = (
     | _SHUTDOWN_CODES
 )
 # From the retryable writes spec.
-_RETRYABLE_ERROR_CODES = _NOT_PRIMARY_CODES | frozenset(
+_RETRYABLE_ERROR_CODES: frozenset = _NOT_PRIMARY_CODES | frozenset(
     [
         7,  # HostNotFound
         6,  # HostUnreachable
@@ -66,10 +68,13 @@ _RETRYABLE_ERROR_CODES = _NOT_PRIMARY_CODES | frozenset(
     ]
 )
 
+# Server code raised when re-authentication is required
+_REAUTHENTICATION_REQUIRED_CODE = 391
+
 
 def _gen_index_name(keys):
     """Generate an index name from the set of fields it is over."""
-    return "_".join(["%s_%s" % item for item in keys])
+    return "_".join(["{}_{}".format(*item) for item in keys])
 
 
 def _index_list(key_or_list, direction=None):
@@ -78,6 +83,8 @@ def _index_list(key_or_list, direction=None):
     Takes such a list, or a single key, or a single key and direction.
     """
     if direction is not None:
+        if not isinstance(key_or_list, str):
+            raise TypeError("Expected a string and a direction")
         return [(key_or_list, direction)]
     else:
         if isinstance(key_or_list, str):
@@ -85,10 +92,13 @@ def _index_list(key_or_list, direction=None):
         if isinstance(key_or_list, abc.ItemsView):
             return list(key_or_list)
         elif not isinstance(key_or_list, (list, tuple)):
-            raise TypeError(
-                "if no direction is specified, " "key_or_list must be an instance of list"
-            )
-        return key_or_list
+            raise TypeError("if no direction is specified, key_or_list must be an instance of list")
+        values = []
+        for item in key_or_list:
+            if isinstance(item, str):
+                item = (item, ASCENDING)
+            values.append(item)
+        return values
 
 
 def _index_document(index_list):
@@ -103,12 +113,15 @@ def _index_document(index_list):
             "mean %r?" % list(index_list.items())
         )
     elif not isinstance(index_list, (list, tuple)):
-        raise TypeError("must use a list of (key, direction) pairs, " "not: " + repr(index_list))
+        raise TypeError("must use a list of (key, direction) pairs, not: " + repr(index_list))
     if not len(index_list):
         raise ValueError("key_or_list must not be the empty list")
 
-    index = SON()
-    for (key, value) in index_list:
+    index: SON[str, Any] = SON()
+    for item in index_list:
+        if isinstance(item, str):
+            item = (item, ASCENDING)
+        key, value = item
         if not isinstance(key, str):
             raise TypeError("first item in each key pair must be an instance of str")
         if not isinstance(value, (str, int, abc.Mapping)):
@@ -181,7 +194,7 @@ def _check_command_response(
     raise OperationFailure(errmsg, code, response, max_wire_version)
 
 
-def _raise_last_write_error(write_errors):
+def _raise_last_write_error(write_errors: List[Any]) -> NoReturn:
     # If the last batch had multiple errors only report
     # the last error to emulate continue_on_error.
     error = write_errors[-1]
@@ -190,8 +203,8 @@ def _raise_last_write_error(write_errors):
     raise WriteError(error.get("errmsg"), error.get("code"), error)
 
 
-def _raise_write_concern_error(error):
-    if "errInfo" in error and error["errInfo"].get("wtimeout"):
+def _raise_write_concern_error(error: Any) -> NoReturn:
+    if _wtimeout_error(error):
         # Make sure we raise WTimeoutError
         raise WTimeoutError(error.get("errmsg"), error.get("code"), error)
     raise WriteConcernError(error.get("errmsg"), error.get("code"), error)
@@ -235,12 +248,10 @@ def _fields_list_to_dict(fields, option_name):
 
     if isinstance(fields, (abc.Sequence, abc.Set)):
         if not all(isinstance(field, str) for field in fields):
-            raise TypeError(
-                "%s must be a list of key names, each an " "instance of str" % (option_name,)
-            )
+            raise TypeError(f"{option_name} must be a list of key names, each an instance of str")
         return dict.fromkeys(fields, 1)
 
-    raise TypeError("%s must be a mapping or " "list of key names" % (option_name,))
+    raise TypeError(f"{option_name} must be a mapping or list of key names")
 
 
 def _handle_exception():
@@ -253,7 +264,39 @@ def _handle_exception():
         einfo = sys.exc_info()
         try:
             traceback.print_exception(einfo[0], einfo[1], einfo[2], None, sys.stderr)
-        except IOError:
+        except OSError:
             pass
         finally:
             del einfo
+
+
+def _handle_reauth(func):
+    def inner(*args, **kwargs):
+        no_reauth = kwargs.pop("no_reauth", False)
+        from pymongo.pool import SocketInfo
+
+        try:
+            return func(*args, **kwargs)
+        except OperationFailure as exc:
+            if no_reauth:
+                raise
+            if exc.code == _REAUTHENTICATION_REQUIRED_CODE:
+                # Look for an argument that either is a SocketInfo
+                # or has a socket_info attribute, so we can trigger
+                # a reauth.
+                sock_info = None
+                for arg in args:
+                    if isinstance(arg, SocketInfo):
+                        sock_info = arg
+                        break
+                    if hasattr(arg, "sock_info"):
+                        sock_info = arg.sock_info
+                        break
+                if sock_info:
+                    sock_info.authenticate(reauthenticate=True)
+                else:
+                    raise
+                return func(*args, **kwargs)
+            raise
+
+    return inner
