@@ -15,12 +15,10 @@
 """Represent a response from the server."""
 
 
-class Response(object):
-    __slots__ = ('_data', '_address', '_request_id', '_duration',
-                 '_from_command', '_docs')
+class Response:
+    __slots__ = ("_data", "_address", "_request_id", "_duration", "_from_command", "_docs")
 
-    def __init__(self, data, address, request_id, duration, from_command,
-                 docs):
+    def __init__(self, data, address, request_id, duration, from_command, docs):
         """Represent a response from the server.
 
         :Parameters:
@@ -67,11 +65,13 @@ class Response(object):
         """The decoded document(s)."""
         return self._docs
 
-class ExhaustResponse(Response):
-    __slots__ = ('_socket_info', '_pool')
 
-    def __init__(self, data, address, socket_info, pool, request_id, duration,
-                 from_command, docs):
+class PinnedResponse(Response):
+    __slots__ = ("_socket_info", "_more_to_come")
+
+    def __init__(
+        self, data, address, socket_info, request_id, duration, from_command, docs, more_to_come
+    ):
         """Represent a response to an exhaust cursor's initial query.
 
         :Parameters:
@@ -82,14 +82,13 @@ class ExhaustResponse(Response):
           - `request_id`: The request id of this operation.
           - `duration`: The duration of the operation.
           - `from_command`: If the response is the result of a db command.
+          - `docs`: List of documents.
+          - `more_to_come`: Bool indicating whether cursor is ready to be
+            exhausted.
         """
-        super(ExhaustResponse, self).__init__(data,
-                                              address,
-                                              request_id,
-                                              duration,
-                                              from_command, docs)
+        super().__init__(data, address, request_id, duration, from_command, docs)
         self._socket_info = socket_info
-        self._pool = pool
+        self._more_to_come = more_to_come
 
     @property
     def socket_info(self):
@@ -102,6 +101,8 @@ class ExhaustResponse(Response):
         return self._socket_info
 
     @property
-    def pool(self):
-        """The Pool from which the SocketInfo came."""
-        return self._pool
+    def more_to_come(self):
+        """If true, server is ready to send batches on the socket until the
+        result set is exhausted or there is an error.
+        """
+        return self._more_to_come
