@@ -42,7 +42,7 @@ DEFAULT_PORT = 27017
 def _unquoted_percent(s):
     """Check for unescaped percent signs.
 
-    :Paramaters:
+    :Parameters:
         - `s`: A string. `s` can have things like '%25', '%2525',
            and '%E2%85%A8' but cannot have unquoted percent like '%foo'.
     """
@@ -64,7 +64,7 @@ def parse_userinfo(userinfo: str) -> Tuple[str, str]:
     Returns a 2-tuple containing the unescaped username followed
     by the unescaped password.
 
-    :Paramaters:
+    :Parameters:
         - `userinfo`: A string of the form <username>:<password>
     """
     if "@" in userinfo or userinfo.count(":") > 1 or _unquoted_percent(userinfo):
@@ -134,7 +134,7 @@ def parse_host(entity: str, default_port: Optional[int] = DEFAULT_PORT) -> _Addr
         host, port = host.split(":", 1)
     if isinstance(port, str):
         if not port.isdigit() or int(port) > 65535 or int(port) <= 0:
-            raise ValueError("Port must be an integer between 0 and 65535: %r" % (port,))
+            raise ValueError(f"Port must be an integer between 0 and 65535: {port!r}")
         port = int(port)
 
     # Normalize hostname to lowercase, since DNS is case-insensitive:
@@ -155,7 +155,8 @@ _IMPLICIT_TLSINSECURE_OPTS = {
 def _parse_options(opts, delim):
     """Helper method for split_options which creates the options dict.
     Also handles the creation of a list for the URI tag_sets/
-    readpreferencetags portion, and the use of a unicode options string."""
+    readpreferencetags portion, and the use of a unicode options string.
+    """
     options = _CaseInsensitiveDictionary()
     for uriopt in opts.split(delim):
         key, value = uriopt.split("=")
@@ -163,7 +164,7 @@ def _parse_options(opts, delim):
             options.setdefault(key, []).append(value)
         else:
             if key in options:
-                warnings.warn("Duplicate URI option '%s'." % (key,))
+                warnings.warn(f"Duplicate URI option '{key}'.")
             if key.lower() == "authmechanismproperties":
                 val = value
             else:
@@ -469,15 +470,13 @@ def parse_uri(
             raise ConfigurationError(
                 'The "dnspython" module must be '
                 "installed to use mongodb+srv:// URIs. "
-                "To fix this error install pymongo with the srv extra:\n "
-                '%s -m pip install "pymongo[srv]"' % (python_path)
+                "To fix this error install pymongo again:\n "
+                "%s -m pip install pymongo>=4.3" % (python_path)
             )
         is_srv = True
         scheme_free = uri[SRV_SCHEME_LEN:]
     else:
-        raise InvalidURI(
-            "Invalid URI scheme: URI must begin with '%s' or '%s'" % (SCHEME, SRV_SCHEME)
-        )
+        raise InvalidURI(f"Invalid URI scheme: URI must begin with '{SCHEME}' or '{SRV_SCHEME}'")
 
     if not scheme_free:
         raise InvalidURI("Must provide at least one hostname or IP.")
@@ -525,15 +524,13 @@ def parse_uri(
     srv_max_hosts = srv_max_hosts or options.get("srvMaxHosts")
     if is_srv:
         if options.get("directConnection"):
-            raise ConfigurationError(
-                "Cannot specify directConnection=true with %s URIs" % (SRV_SCHEME,)
-            )
+            raise ConfigurationError(f"Cannot specify directConnection=true with {SRV_SCHEME} URIs")
         nodes = split_hosts(hosts, default_port=None)
         if len(nodes) != 1:
-            raise InvalidURI("%s URIs must include one, and only one, hostname" % (SRV_SCHEME,))
+            raise InvalidURI(f"{SRV_SCHEME} URIs must include one, and only one, hostname")
         fqdn, port = nodes[0]
         if port is not None:
-            raise InvalidURI("%s URIs must not include a port number" % (SRV_SCHEME,))
+            raise InvalidURI(f"{SRV_SCHEME} URIs must not include a port number")
 
         # Use the connection timeout. connectTimeoutMS passed as a keyword
         # argument overrides the same option passed in the connection string.
@@ -605,7 +602,6 @@ def _parse_kms_tls_options(kms_tls_options):
             "tlsInsecure",
             "tlsAllowInvalidCertificates",
             "tlsAllowInvalidHostnames",
-            "tlsDisableOCSPEndpointCheck",
             "tlsDisableCertificateRevocationCheck",
         ]:
             if n in opts:
