@@ -28,6 +28,7 @@ import sys
 import threading
 import time
 from typing import Iterable, Type, no_type_check
+from unittest.mock import patch
 
 sys.path[0:0] = [""]
 
@@ -65,7 +66,12 @@ from test.utils import (
 
 import pymongo
 from bson import encode
-from bson.codec_options import CodecOptions, TypeEncoder, TypeRegistry
+from bson.codec_options import (
+    CodecOptions,
+    DatetimeConversion,
+    TypeEncoder,
+    TypeRegistry,
+)
 from bson.son import SON
 from bson.tz_util import utc
 from pymongo import event_loggers, message, monitoring
@@ -108,7 +114,6 @@ class ClientUnitTest(unittest.TestCase):
     client: MongoClient
 
     @classmethod
-    @client_context.require_connection
     def setUpClass(cls):
         cls.client = rs_or_single_client(connect=False, serverSelectionTimeoutMS=100)
 
@@ -212,7 +217,7 @@ class ClientUnitTest(unittest.TestCase):
 
     def test_iteration(self):
         client = self.client
-        if "PyPy" in sys.version:
+        if "PyPy" in sys.version and sys.version_info < (3, 8, 15):
             msg = "'NoneType' object is not callable"
         else:
             msg = "'MongoClient' object is not iterable"
@@ -320,7 +325,7 @@ class ClientUnitTest(unittest.TestCase):
         self.assertRaises(TypeError, MongoClient, driver=("Foo", "1", "a"))
         # Test appending to driver info.
         metadata["driver"]["name"] = "PyMongo|FooDriver"
-        metadata["driver"]["version"] = "%s|1.2.3" % (_METADATA["driver"]["version"],)
+        metadata["driver"]["version"] = "{}|1.2.3".format(_METADATA["driver"]["version"])
         client = MongoClient(
             "foo",
             27017,
@@ -330,7 +335,7 @@ class ClientUnitTest(unittest.TestCase):
         )
         options = client._MongoClient__options
         self.assertEqual(options.pool_options.metadata, metadata)
-        metadata["platform"] = "%s|FooPlatform" % (_METADATA["platform"],)
+        metadata["platform"] = "{}|FooPlatform".format(_METADATA["platform"])
         client = MongoClient(
             "foo",
             27017,
@@ -342,7 +347,7 @@ class ClientUnitTest(unittest.TestCase):
         self.assertEqual(options.pool_options.metadata, metadata)
 
     def test_kwargs_codec_options(self):
-        class MyFloatType(object):
+        class MyFloatType:
             def __init__(self, x):
                 self.__x = x
 
@@ -386,14 +391,17 @@ class ClientUnitTest(unittest.TestCase):
         # Ensure codec options are passed in correctly
         uuid_representation_label = "javaLegacy"
         unicode_decode_error_handler = "ignore"
+        datetime_conversion = "DATETIME_CLAMP"
         uri = (
             "mongodb://%s:%d/foo?tz_aware=true&uuidrepresentation="
             "%s&unicode_decode_error_handler=%s"
+            "&datetime_conversion=%s"
             % (
                 client_context.host,
                 client_context.port,
                 uuid_representation_label,
                 unicode_decode_error_handler,
+                datetime_conversion,
             )
         )
         c = MongoClient(uri, connect=False)
@@ -403,6 +411,17 @@ class ClientUnitTest(unittest.TestCase):
             c.codec_options.uuid_representation, _UUID_REPRESENTATIONS[uuid_representation_label]
         )
         self.assertEqual(c.codec_options.unicode_decode_error_handler, unicode_decode_error_handler)
+        self.assertEqual(
+            c.codec_options.datetime_conversion, DatetimeConversion[datetime_conversion]
+        )
+
+        # Change the passed datetime_conversion to a number and re-assert.
+        uri = uri.replace(datetime_conversion, f"{int(DatetimeConversion[datetime_conversion])}")
+        c = MongoClient(uri, connect=False)
+
+        self.assertEqual(
+            c.codec_options.datetime_conversion, DatetimeConversion[datetime_conversion]
+        )
 
     def test_uri_option_precedence(self):
         # Ensure kwarg options override connection string options.
@@ -685,7 +704,7 @@ class TestClient(IntegrationTest):
         self.assertRaises(ConnectionFailure, c.pymongo_test.test.find_one)
 
     def test_equality(self):
-        seed = "%s:%s" % list(self.client._topology_settings.seeds)[0]
+        seed = "{}:{}".format(*list(self.client._topology_settings.seeds)[0])
         c = rs_or_single_client(seed, connect=False)
         self.addCleanup(c.close)
         self.assertEqual(client_context.client, c)
@@ -704,7 +723,7 @@ class TestClient(IntegrationTest):
         )
 
     def test_hashable(self):
-        seed = "%s:%s" % list(self.client._topology_settings.seeds)[0]
+        seed = "{}:{}".format(*list(self.client._topology_settings.seeds)[0])
         c = rs_or_single_client(seed, connect=False)
         self.addCleanup(c.close)
         self.assertIn(c, {client_context.client})
@@ -716,7 +735,7 @@ class TestClient(IntegrationTest):
         with self.assertRaises(ValueError):
             connected(
                 MongoClient(
-                    "%s:1234567" % (client_context.host,),
+                    f"{client_context.host}:1234567",
                     connectTimeoutMS=1,
                     serverSelectionTimeoutMS=10,
                 )
@@ -772,6 +791,10 @@ class TestClient(IntegrationTest):
         self.assertIsInstance(cursor, CommandCursor)
         helper_docs = list(cursor)
         self.assertTrue(len(helper_docs) > 0)
+        # sizeOnDisk can change between calls.
+        for doc_list in (helper_docs, cmd_docs):
+            for doc in doc_list:
+                doc.pop("sizeOnDisk", None)
         self.assertEqual(helper_docs, cmd_docs)
         for doc in helper_docs:
             self.assertIs(type(doc), dict)
@@ -979,7 +1002,7 @@ class TestClient(IntegrationTest):
     @client_context.require_auth
     def test_lazy_auth_raises_operation_failure(self):
         lazy_client = rs_or_single_client_noauth(
-            "mongodb://user:wrong@%s/pymongo_test" % (client_context.host,), connect=False
+            f"mongodb://user:wrong@{client_context.host}/pymongo_test", connect=False
         )
 
         assertRaisesExactly(OperationFailure, lazy_client.test.collection.find_one)
@@ -1120,7 +1143,7 @@ class TestClient(IntegrationTest):
         naive = self.client
         aware.pymongo_test.drop_collection("test")
 
-        now = datetime.datetime.utcnow()
+        now = datetime.datetime.now(tz=datetime.timezone.utc)
         aware.pymongo_test.test.insert_one({"x": now})
 
         self.assertEqual(None, naive.pymongo_test.test.find_one()["x"].tzinfo)
@@ -1137,7 +1160,7 @@ class TestClient(IntegrationTest):
                 raise SkipTest("Need the ipaddress module to test with SSL")
 
         if client_context.auth_enabled:
-            auth_str = "%s:%s@" % (db_user, db_pwd)
+            auth_str = f"{db_user}:{db_pwd}@"
         else:
             auth_str = ""
 
@@ -1510,7 +1533,7 @@ class TestClient(IntegrationTest):
         # Continuously reset the pool.
         class ResetPoolThread(threading.Thread):
             def __init__(self, pool):
-                super(ResetPoolThread, self).__init__()
+                super().__init__()
                 self.running = True
                 self.pool = pool
 
@@ -1596,7 +1619,6 @@ class TestClient(IntegrationTest):
         with self.assertRaises(ConfigurationError):
             MongoClient(["host1", "host2"], directConnection=True)
 
-    @unittest.skipIf(sys.platform.startswith("java"), "Jython does not support gc.get_objects")
     @unittest.skipIf("PyPy" in sys.version, "PYTHON-2927 fails often on PyPy")
     def test_continuous_network_errors(self):
         def server_description_count():
@@ -1612,7 +1634,7 @@ class TestClient(IntegrationTest):
         gc.collect()
         with client_knobs(min_heartbeat_interval=0.003):
             client = MongoClient(
-                "invalid:27017", heartbeatFrequencyMS=3, serverSelectionTimeoutMS=100
+                "invalid:27017", heartbeatFrequencyMS=3, serverSelectionTimeoutMS=150
             )
             initial_count = server_description_count()
             self.addCleanup(client.close)
@@ -1622,8 +1644,9 @@ class TestClient(IntegrationTest):
             final_count = server_description_count()
             # If a bug like PYTHON-2433 is reintroduced then too many
             # ServerDescriptions will be kept alive and this test will fail:
-            # AssertionError: 4 != 22 within 5 delta (18 difference)
-            self.assertAlmostEqual(initial_count, final_count, delta=10)
+            # AssertionError: 19 != 46 within 15 delta (27 difference)
+            # On Python 3.11 we seem to get more of a delta.
+            self.assertAlmostEqual(initial_count, final_count, delta=20)
 
     @client_context.require_failCommand_fail_point
     def test_network_error_message(self):
@@ -1633,7 +1656,8 @@ class TestClient(IntegrationTest):
         with self.fail_point(
             {"mode": {"times": 1}, "data": {"closeConnection": True, "failCommands": ["find"]}}
         ):
-            expected = "%s:%s: " % client.address
+            assert client.address is not None
+            expected = "{}:{}: ".format(*client.address)
             with self.assertRaisesRegex(AutoReconnect, expected):
                 client.pymongo_test.test.find_one({})
 
@@ -1727,12 +1751,92 @@ class TestClient(IntegrationTest):
         self.assertIn("TEST COMPLETED", log_output)
         self.assertNotIn("ServerHeartbeatFailedEvent", log_output)
 
+    def _test_handshake(self, env_vars, expected_env):
+        with patch.dict("os.environ", env_vars):
+            metadata = copy.deepcopy(_METADATA)
+            if expected_env is not None:
+                metadata["env"] = expected_env
+            with rs_or_single_client(serverSelectionTimeoutMS=10000) as client:
+                client.admin.command("ping")
+                options = client._MongoClient__options
+                self.assertEqual(options.pool_options.metadata, metadata)
+
+    def test_handshake_01_aws(self):
+        self._test_handshake(
+            {
+                "AWS_EXECUTION_ENV": "AWS_Lambda_python3.9",
+                "AWS_REGION": "us-east-2",
+                "AWS_LAMBDA_FUNCTION_MEMORY_SIZE": "1024",
+            },
+            {"name": "aws.lambda", "region": "us-east-2", "memory_mb": 1024},
+        )
+
+    def test_handshake_02_azure(self):
+        self._test_handshake({"FUNCTIONS_WORKER_RUNTIME": "python"}, {"name": "azure.func"})
+
+    def test_handshake_03_gcp(self):
+        self._test_handshake(
+            {
+                "K_SERVICE": "servicename",
+                "FUNCTION_MEMORY_MB": "1024",
+                "FUNCTION_TIMEOUT_SEC": "60",
+                "FUNCTION_REGION": "us-central1",
+            },
+            {"name": "gcp.func", "region": "us-central1", "memory_mb": 1024, "timeout_sec": 60},
+        )
+        # Extra case for FUNCTION_NAME.
+        self._test_handshake(
+            {
+                "FUNCTION_NAME": "funcname",
+                "FUNCTION_MEMORY_MB": "1024",
+                "FUNCTION_TIMEOUT_SEC": "60",
+                "FUNCTION_REGION": "us-central1",
+            },
+            {"name": "gcp.func", "region": "us-central1", "memory_mb": 1024, "timeout_sec": 60},
+        )
+
+    def test_handshake_04_vercel(self):
+        self._test_handshake(
+            {"VERCEL": "1", "VERCEL_REGION": "cdg1"}, {"name": "vercel", "region": "cdg1"}
+        )
+
+    def test_handshake_05_multiple(self):
+        self._test_handshake(
+            {"AWS_EXECUTION_ENV": "AWS_Lambda_python3.9", "FUNCTIONS_WORKER_RUNTIME": "python"},
+            None,
+        )
+        # Extra cases for other combos.
+        self._test_handshake(
+            {"FUNCTIONS_WORKER_RUNTIME": "python", "K_SERVICE": "servicename"},
+            None,
+        )
+        self._test_handshake({"K_SERVICE": "servicename", "VERCEL": "1"}, None)
+
+    def test_handshake_06_region_too_long(self):
+        self._test_handshake(
+            {"AWS_EXECUTION_ENV": "AWS_Lambda_python3.9", "AWS_REGION": "a" * 512},
+            {"name": "aws.lambda"},
+        )
+
+    def test_handshake_07_memory_invalid_int(self):
+        self._test_handshake(
+            {"AWS_EXECUTION_ENV": "AWS_Lambda_python3.9", "AWS_LAMBDA_FUNCTION_MEMORY_SIZE": "big"},
+            {"name": "aws.lambda"},
+        )
+
+    def test_handshake_08_invalid_aws_ec2(self):
+        # AWS_EXECUTION_ENV needs to start with "AWS_Lambda_".
+        self._test_handshake(
+            {"AWS_EXECUTION_ENV": "EC2"},
+            None,
+        )
+
 
 class TestExhaustCursor(IntegrationTest):
     """Test that clients properly handle errors from exhaust cursors."""
 
     def setUp(self):
-        super(TestExhaustCursor, self).setUp()
+        super().setUp()
         if client_context.is_mongos:
             raise SkipTest("mongos doesn't support exhaust, SERVER-2627")
 
@@ -1843,6 +1947,87 @@ class TestExhaustCursor(IntegrationTest):
         self.assertNotIn(sock_info, pool.sockets)
         self.assertEqual(0, pool.requests)
 
+    def test_gevent_task(self):
+        if not gevent_monkey_patched():
+            raise SkipTest("Must be running monkey patched by gevent")
+        from gevent import spawn
+
+        def poller():
+            while True:
+                client_context.client.pymongo_test.test.insert_one({})
+
+        task = spawn(poller)
+        task.kill()
+        self.assertTrue(task.dead)
+
+    def test_gevent_timeout(self):
+        if not gevent_monkey_patched():
+            raise SkipTest("Must be running monkey patched by gevent")
+        from gevent import Timeout, spawn
+
+        client = rs_or_single_client(maxPoolSize=1)
+        coll = client.pymongo_test.test
+        coll.insert_one({})
+
+        def contentious_task():
+            # The 10 second timeout causes this test to fail without blocking
+            # forever if a bug like PYTHON-2334 is reintroduced.
+            with Timeout(10):
+                coll.find_one({"$where": delay(1)})
+
+        def timeout_task():
+            with Timeout(0.5):
+                try:
+                    coll.find_one({})
+                except Timeout:
+                    pass
+
+        ct = spawn(contentious_task)
+        tt = spawn(timeout_task)
+        tt.join(15)
+        ct.join(15)
+        self.assertTrue(tt.dead)
+        self.assertTrue(ct.dead)
+        self.assertIsNone(tt.get())
+        self.assertIsNone(ct.get())
+
+    def test_gevent_timeout_when_creating_connection(self):
+        if not gevent_monkey_patched():
+            raise SkipTest("Must be running monkey patched by gevent")
+        from gevent import Timeout, spawn
+
+        client = rs_or_single_client()
+        self.addCleanup(client.close)
+        coll = client.pymongo_test.test
+        pool = get_pool(client)
+
+        # Patch the pool to delay the connect method.
+        def delayed_connect(*args, **kwargs):
+            time.sleep(3)
+            return pool.__class__.connect(pool, *args, **kwargs)
+
+        pool.connect = delayed_connect
+
+        def timeout_task():
+            with Timeout(1):
+                try:
+                    coll.find_one({})
+                    return False
+                except Timeout:
+                    return True
+
+        tt = spawn(timeout_task)
+        tt.join(10)
+
+        # Assert that we got our active_sockets count back
+        self.assertEqual(pool.active_sockets, 0)
+        # Assert the greenlet is dead
+        self.assertTrue(tt.dead)
+        # Assert that the Timeout was raised all the way to the try
+        self.assertTrue(tt.get())
+        # Unpatch the instance.
+        del pool.connect
+
 
 class TestClientLazyConnect(IntegrationTest):
     """Test concurrent operations on a lazily-connecting MongoClient."""
@@ -2003,106 +2188,35 @@ class TestMongoClientFailover(MockClientTest):
             self.assertEqual(7, sd_b.max_wire_version)
 
     def test_network_error_on_query(self):
-        callback = lambda client: client.db.collection.find_one()
+        def callback(client):
+            return client.db.collection.find_one()
+
         self._test_network_error(callback)
 
     def test_network_error_on_insert(self):
-        callback = lambda client: client.db.collection.insert_one({})
+        def callback(client):
+            return client.db.collection.insert_one({})
+
         self._test_network_error(callback)
 
     def test_network_error_on_update(self):
-        callback = lambda client: client.db.collection.update_one({}, {"$unset": "x"})
+        def callback(client):
+            return client.db.collection.update_one({}, {"$unset": "x"})
+
         self._test_network_error(callback)
 
     def test_network_error_on_replace(self):
-        callback = lambda client: client.db.collection.replace_one({}, {})
+        def callback(client):
+            return client.db.collection.replace_one({}, {})
+
         self._test_network_error(callback)
 
     def test_network_error_on_delete(self):
-        callback = lambda client: client.db.collection.delete_many({})
+        def callback(client):
+            return client.db.collection.delete_many({})
+
         self._test_network_error(callback)
 
-    def test_gevent_task(self):
-        if not gevent_monkey_patched():
-            raise SkipTest("Must be running monkey patched by gevent")
-        from gevent import spawn
-
-        def poller():
-            while True:
-                client_context.client.pymongo_test.test.insert_one({})
-
-        task = spawn(poller)
-        task.kill()
-        self.assertTrue(task.dead)
-
-    def test_gevent_timeout(self):
-        if not gevent_monkey_patched():
-            raise SkipTest("Must be running monkey patched by gevent")
-        from gevent import Timeout, spawn
-
-        client = rs_or_single_client(maxPoolSize=1)
-        coll = client.pymongo_test.test
-        coll.insert_one({})
-
-        def contentious_task():
-            # The 10 second timeout causes this test to fail without blocking
-            # forever if a bug like PYTHON-2334 is reintroduced.
-            with Timeout(10):
-                coll.find_one({"$where": delay(1)})
-
-        def timeout_task():
-            with Timeout(0.5):
-                try:
-                    coll.find_one({})
-                except Timeout:
-                    pass
-
-        ct = spawn(contentious_task)
-        tt = spawn(timeout_task)
-        tt.join(15)
-        ct.join(15)
-        self.assertTrue(tt.dead)
-        self.assertTrue(ct.dead)
-        self.assertIsNone(tt.get())
-        self.assertIsNone(ct.get())
-
-    def test_gevent_timeout_when_creating_connection(self):
-        if not gevent_monkey_patched():
-            raise SkipTest("Must be running monkey patched by gevent")
-        from gevent import Timeout, spawn
-
-        client = rs_or_single_client()
-        self.addCleanup(client.close)
-        coll = client.pymongo_test.test
-        pool = get_pool(client)
-
-        # Patch the pool to delay the connect method.
-        def delayed_connect(*args, **kwargs):
-            time.sleep(3)
-            return pool.__class__.connect(pool, *args, **kwargs)
-
-        pool.connect = delayed_connect
-
-        def timeout_task():
-            with Timeout(1):
-                try:
-                    coll.find_one({})
-                    return False
-                except Timeout:
-                    return True
-
-        tt = spawn(timeout_task)
-        tt.join(10)
-
-        # Assert that we got our active_sockets count back
-        self.assertEqual(pool.active_sockets, 0)
-        # Assert the greenlet is dead
-        self.assertTrue(tt.dead)
-        # Assert that the Timeout was raised all the way to the try
-        self.assertTrue(tt.get())
-        # Unpatch the instance.
-        del pool.connect
-
 
 class TestClientPool(MockClientTest):
     @client_context.require_connection
@@ -2123,7 +2237,7 @@ class TestClientPool(MockClientTest):
 
         wait_until(lambda: len(c.nodes) == 3, "connect")
         self.assertEqual(c.address, ("a", 1))
-        self.assertEqual(c.arbiters, set([("c", 3)]))
+        self.assertEqual(c.arbiters, {("c", 3)})
         # Assert that we create 2 and only 2 pooled connections.
         listener.wait_for_event(monitoring.ConnectionReadyEvent, 2)
         self.assertEqual(listener.event_count(monitoring.ConnectionCreatedEvent), 2)
