@@ -1,5 +1,3 @@
-# -*- coding: utf-8 -*-
-
 # Copyright 2009-present MongoDB, Inc.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
@@ -21,6 +19,9 @@ import re
 import sys
 from codecs import utf_8_decode
 from collections import defaultdict
+from typing import Iterable, no_type_check
+
+from pymongo.database import Database
 
 sys.path[0:0] = [""]
 
@@ -75,6 +76,8 @@ from pymongo.write_concern import WriteConcern
 class TestCollectionNoConnect(unittest.TestCase):
     """Test Collection features on a client that does not connect."""
 
+    db: Database
+
     @classmethod
     def setUpClass(cls):
         cls.db = MongoClient(connect=False).pymongo_test
@@ -119,14 +122,35 @@ class TestCollectionNoConnect(unittest.TestCase):
         self.assertEqual(coll2.write_concern, coll4.write_concern)
 
     def test_iteration(self):
-        self.assertRaises(TypeError, next, self.db)
+        coll = self.db.coll
+        if "PyPy" in sys.version and sys.version_info < (3, 8, 15):
+            msg = "'NoneType' object is not callable"
+        else:
+            msg = "'Collection' object is not iterable"
+        # Iteration fails
+        with self.assertRaisesRegex(TypeError, msg):
+            for _ in coll:  # type: ignore[misc] # error: "None" not callable  [misc]
+                break
+        # Non-string indices will start failing in PyMongo 5.
+        self.assertEqual(coll[0].name, "coll.0")
+        self.assertEqual(coll[{}].name, "coll.{}")
+        # next fails
+        with self.assertRaisesRegex(TypeError, "'Collection' object is not iterable"):
+            _ = next(coll)
+        # .next() fails
+        with self.assertRaisesRegex(TypeError, "'Collection' object is not iterable"):
+            _ = coll.next()
+        # Do not implement typing.Iterable.
+        self.assertNotIsInstance(coll, Iterable)
 
 
 class TestCollection(IntegrationTest):
+    w: int
+
     @classmethod
     def setUpClass(cls):
-        super(TestCollection, cls).setUpClass()
-        cls.w = client_context.w
+        super().setUpClass()
+        cls.w = client_context.w  # type: ignore
 
     @classmethod
     def tearDownClass(cls):
@@ -239,8 +263,7 @@ class TestCollection(IntegrationTest):
         db = self.db
         with self.assertRaisesRegex(
             ConfigurationError,
-            "Must be connected to MongoDB 4\.4\+ to use the commitQuorum "
-            "option for createIndexes",
+            r"Must be connected to MongoDB 4\.4\+ to use the commitQuorum option for createIndexes",
         ):
             db.coll.create_indexes([IndexModel("a")], commitQuorum="majority")
 
@@ -282,6 +305,10 @@ class TestCollection(IntegrationTest):
         db.test.create_index([("hello", DESCENDING), ("world", ASCENDING)])
         self.assertTrue("hello_-1_world_1" in db.test.index_information())
 
+        db.test.drop_indexes()
+        db.test.create_index([("hello", DESCENDING), ("world", ASCENDING)], name=None)
+        self.assertTrue("hello_-1_world_1" in db.test.index_information())
+
         db.test.drop()
         db.test.insert_one({"a": 1})
         db.test.insert_one({"a": 1})
@@ -290,6 +317,10 @@ class TestCollection(IntegrationTest):
         with self.write_concern_collection() as coll:
             coll.create_index([("hello", DESCENDING)])
 
+        db.test.create_index(["hello", "world"])
+        db.test.create_index(["hello", ("world", DESCENDING)])
+        db.test.create_index({"hello": 1}.items())  # type:ignore[arg-type]
+
     def test_drop_index(self):
         db = self.db
         db.test.drop_indexes()
@@ -340,7 +371,7 @@ class TestCollection(IntegrationTest):
         db.test.insert_one({})  # create collection
 
         def map_indexes(indexes):
-            return dict([(index["name"], index) for index in indexes])
+            return {index["name"]: index for index in indexes}
 
         indexes = list(db.test.list_indexes())
         self.assertEqual(len(indexes), 1)
@@ -452,7 +483,7 @@ class TestCollection(IntegrationTest):
         db.test.drop_indexes()
         self.assertEqual("geo_2dsphere", db.test.create_index([("geo", GEOSPHERE)]))
 
-        for dummy, info in db.test.index_information().items():
+        for _dummy, info in db.test.index_information().items():
             field, idx_type = info["key"][0]
             if field == "geo" and idx_type == "2dsphere":
                 break
@@ -471,7 +502,7 @@ class TestCollection(IntegrationTest):
         db.test.drop_indexes()
         self.assertEqual("a_hashed", db.test.create_index([("a", HASHED)]))
 
-        for dummy, info in db.test.index_information().items():
+        for _dummy, info in db.test.index_information().items():
             field, idx_type = info["key"][0]
             if field == "a" and idx_type == "hashed":
                 break
@@ -653,8 +684,6 @@ class TestCollection(IntegrationTest):
         db.drop_collection("test")
         db.create_collection("test", capped=True, size=4096)
         result = db.test.options()
-        # mongos 2.2.x adds an $auth field when auth is enabled.
-        result.pop("$auth", None)
         self.assertEqual(result, {"capped": True, "size": 4096})
         db.drop_collection("test")
 
@@ -698,7 +727,7 @@ class TestCollection(IntegrationTest):
         db = self.db
         db.test.drop()
 
-        docs = [{} for _ in range(5)]
+        docs: list = [{} for _ in range(5)]
         result = db.test.insert_many(docs)
         self.assertTrue(isinstance(result, InsertManyResult))
         self.assertTrue(isinstance(result.inserted_ids, list))
@@ -729,7 +758,7 @@ class TestCollection(IntegrationTest):
         self.assertEqual([], result.inserted_ids)
 
         db = db.client.get_database(db.name, write_concern=WriteConcern(w=0))
-        docs = [{} for _ in range(5)]
+        docs: list = [{} for _ in range(5)]
         result = db.test.insert_many(docs)
         self.assertTrue(isinstance(result, InsertManyResult))
         self.assertFalse(result.acknowledged)
@@ -759,7 +788,7 @@ class TestCollection(IntegrationTest):
             db.test.insert_many([])
 
         with self.assertRaisesRegex(TypeError, "documents must be a non-empty list"):
-            db.test.insert_many(1)
+            db.test.insert_many(1)  # type: ignore[arg-type]
 
         with self.assertRaisesRegex(TypeError, "documents must be a non-empty list"):
             db.test.insert_many(RawBSONDocument(encode({"_id": 2})))
@@ -1002,7 +1031,7 @@ class TestCollection(IntegrationTest):
         db.create_collection("test", validator={"a": {"$gte": 0}})
         db_w0 = self.db.client.get_database(self.db.name, write_concern=WriteConcern(w=0))
 
-        ops = [
+        ops: list = [
             InsertOne({"a": -10}),
             InsertOne({"a": -11}),
             InsertOne({"a": -12}),
@@ -1028,7 +1057,7 @@ class TestCollection(IntegrationTest):
     def test_find_by_default_dct(self):
         db = self.db
         db.test.insert_one({"foo": "bar"})
-        dct = defaultdict(dict, [("foo", "bar")])
+        dct = defaultdict(dict, [("foo", "bar")])  # type: ignore[arg-type]
         self.assertIsNotNone(db.test.find_one(dct))
         self.assertEqual(dct, defaultdict(dict, [("foo", "bar")]))
 
@@ -1057,6 +1086,7 @@ class TestCollection(IntegrationTest):
         doc = next(db.test.find({}, ["mike"]))
         self.assertFalse("extra thing" in doc)
 
+    @no_type_check
     def test_fields_specifier_as_dict(self):
         db = self.db
         db.test.delete_many({})
@@ -1255,7 +1285,7 @@ class TestCollection(IntegrationTest):
         self.assertTrue(result.acknowledged)
         self.assertEqual(1, db.test.count_documents({"y": 1}))
         self.assertEqual(0, db.test.count_documents({"x": 1}))
-        self.assertEqual(db.test.find_one(id1)["y"], 1)
+        self.assertEqual(db.test.find_one(id1)["y"], 1)  # type: ignore
 
         replacement = RawBSONDocument(encode({"_id": id1, "z": 1}))
         result = db.test.replace_one({"y": 1}, replacement, True)
@@ -1266,7 +1296,7 @@ class TestCollection(IntegrationTest):
         self.assertTrue(result.acknowledged)
         self.assertEqual(1, db.test.count_documents({"z": 1}))
         self.assertEqual(0, db.test.count_documents({"y": 1}))
-        self.assertEqual(db.test.find_one(id1)["z"], 1)
+        self.assertEqual(db.test.find_one(id1)["z"], 1)  # type: ignore
 
         result = db.test.replace_one({"x": 2}, {"y": 2}, True)
         self.assertTrue(isinstance(result, UpdateResult))
@@ -1297,7 +1327,7 @@ class TestCollection(IntegrationTest):
         self.assertTrue(result.modified_count in (None, 1))
         self.assertIsNone(result.upserted_id)
         self.assertTrue(result.acknowledged)
-        self.assertEqual(db.test.find_one(id1)["x"], 6)
+        self.assertEqual(db.test.find_one(id1)["x"], 6)  # type: ignore
 
         id2 = db.test.insert_one({"x": 1}).inserted_id
         result = db.test.update_one({"x": 6}, {"$inc": {"x": 1}})
@@ -1306,8 +1336,8 @@ class TestCollection(IntegrationTest):
         self.assertTrue(result.modified_count in (None, 1))
         self.assertIsNone(result.upserted_id)
         self.assertTrue(result.acknowledged)
-        self.assertEqual(db.test.find_one(id1)["x"], 7)
-        self.assertEqual(db.test.find_one(id2)["x"], 1)
+        self.assertEqual(db.test.find_one(id1)["x"], 7)  # type: ignore
+        self.assertEqual(db.test.find_one(id2)["x"], 1)  # type: ignore
 
         result = db.test.update_one({"x": 2}, {"$set": {"y": 1}}, True)
         self.assertTrue(isinstance(result, UpdateResult))
@@ -1396,16 +1426,7 @@ class TestCollection(IntegrationTest):
     def test_acknowledged_delete(self):
         db = self.db
         db.drop_collection("test")
-        db.create_collection("test", capped=True, size=1000)
-
-        db.test.insert_one({"x": 1})
-        self.assertEqual(1, db.test.count_documents({}))
-
-        # Can't remove from capped collection.
-        self.assertRaises(OperationFailure, db.test.delete_one, {"x": 1})
-        db.drop_collection("test")
-        db.test.insert_one({"x": 1})
-        db.test.insert_one({"x": 1})
+        db.test.insert_many([{"x": 1}, {"x": 1}])
         self.assertEqual(2, db.test.delete_many({}).deleted_count)
         self.assertEqual(0, db.test.delete_many({}).deleted_count)
 
@@ -1498,14 +1519,14 @@ class TestCollection(IntegrationTest):
 
         # Test that batchSize is handled properly.
         cursor = db.test.aggregate([], batchSize=5)
-        self.assertEqual(5, len(cursor._CommandCursor__data))
+        self.assertEqual(5, len(cursor._CommandCursor__data))  # type: ignore
         # Force a getMore
-        cursor._CommandCursor__data.clear()
+        cursor._CommandCursor__data.clear()  # type: ignore
         next(cursor)
         # batchSize - 1
-        self.assertEqual(4, len(cursor._CommandCursor__data))
+        self.assertEqual(4, len(cursor._CommandCursor__data))  # type: ignore
         # Exhaust the cursor. There shouldn't be any errors.
-        for doc in cursor:
+        for _doc in cursor:
             pass
 
     def test_aggregation_cursor_alive(self):
@@ -1523,6 +1544,13 @@ class TestCollection(IntegrationTest):
 
             self.assertTrue(cursor.alive)
 
+    def test_invalid_session_parameter(self):
+        def try_invalid_session():
+            with self.db.test.aggregate([], {}):  # type:ignore
+                pass
+
+        self.assertRaisesRegex(ValueError, "must be a ClientSession", try_invalid_session)
+
     def test_large_limit(self):
         db = self.db
         db.drop_collection("test_large_limit")
@@ -1589,6 +1617,7 @@ class TestCollection(IntegrationTest):
         with self.write_concern_collection() as coll:
             coll.rename("foo")
 
+    @no_type_check
     def test_find_one(self):
         db = self.db
         db.drop_collection("test")
@@ -1607,8 +1636,8 @@ class TestCollection(IntegrationTest):
         self.assertTrue("hello" in db.test.find_one(projection=("hello",)))
         self.assertTrue("hello" not in db.test.find_one(projection=("foo",)))
 
-        self.assertTrue("hello" in db.test.find_one(projection=set(["hello"])))
-        self.assertTrue("hello" not in db.test.find_one(projection=set(["foo"])))
+        self.assertTrue("hello" in db.test.find_one(projection={"hello"}))
+        self.assertTrue("hello" not in db.test.find_one(projection={"foo"}))
 
         self.assertTrue("hello" in db.test.find_one(projection=frozenset(["hello"])))
         self.assertTrue("hello" not in db.test.find_one(projection=frozenset(["foo"])))
@@ -1657,7 +1686,7 @@ class TestCollection(IntegrationTest):
 
         self.assertRaises(TypeError, db.test.find, sort=5)
         self.assertRaises(TypeError, db.test.find, sort="hello")
-        self.assertRaises(ValueError, db.test.find, sort=["hello", 1])
+        self.assertRaises(TypeError, db.test.find, sort=["hello", 1])
 
     # TODO doesn't actually test functionality, just that it doesn't blow up
     def test_cursor_timeout(self):
@@ -1881,17 +1910,18 @@ class TestCollection(IntegrationTest):
 
         bad = BadGetAttr([("foo", "bar")])
         c.insert_one({"bad": bad})
-        self.assertEqual("bar", c.find_one()["bad"]["foo"])
+        self.assertEqual("bar", c.find_one()["bad"]["foo"])  # type: ignore
 
     def test_array_filters_validation(self):
         # array_filters must be a list.
         c = self.db.test
         with self.assertRaises(TypeError):
-            c.update_one({}, {"$set": {"a": 1}}, array_filters={})
+            c.update_one({}, {"$set": {"a": 1}}, array_filters={})  # type: ignore[arg-type]
         with self.assertRaises(TypeError):
-            c.update_many({}, {"$set": {"a": 1}}, array_filters={})
+            c.update_many({}, {"$set": {"a": 1}}, array_filters={})  # type: ignore[arg-type]
         with self.assertRaises(TypeError):
-            c.find_one_and_update({}, {"$set": {"a": 1}}, array_filters={})
+            update = {"$set": {"a": 1}}
+            c.find_one_and_update({}, update, array_filters={})  # type: ignore[arg-type]
 
     def test_array_filters_unacknowledged(self):
         c_w0 = self.db.test.with_options(write_concern=WriteConcern(w=0))
@@ -1962,21 +1992,20 @@ class TestCollection(IntegrationTest):
         c_w0 = db.get_collection("test", write_concern=WriteConcern(w=0))
         # default WriteConcern.
         c_default = db.get_collection("test", write_concern=WriteConcern())
-        results = listener.results
         # Authenticate the client and throw out auth commands from the listener.
         db.command("ping")
-        results.clear()
+        listener.reset()
         c_w0.find_one_and_update({"_id": 1}, {"$set": {"foo": "bar"}})
-        self.assertEqual({"w": 0}, results["started"][0].command["writeConcern"])
-        results.clear()
+        self.assertEqual({"w": 0}, listener.started_events[0].command["writeConcern"])
+        listener.reset()
 
         c_w0.find_one_and_replace({"_id": 1}, {"foo": "bar"})
-        self.assertEqual({"w": 0}, results["started"][0].command["writeConcern"])
-        results.clear()
+        self.assertEqual({"w": 0}, listener.started_events[0].command["writeConcern"])
+        listener.reset()
 
         c_w0.find_one_and_delete({"_id": 1})
-        self.assertEqual({"w": 0}, results["started"][0].command["writeConcern"])
-        results.clear()
+        self.assertEqual({"w": 0}, listener.started_events[0].command["writeConcern"])
+        listener.reset()
 
         # Test write concern errors.
         if client_context.is_rs:
@@ -1993,27 +2022,27 @@ class TestCollection(IntegrationTest):
                 WriteConcernError,
                 c_wc_error.find_one_and_replace,
                 {"w": 0},
-                results["started"][0].command["writeConcern"],
+                listener.started_events[0].command["writeConcern"],
             )
             self.assertRaises(
                 WriteConcernError,
                 c_wc_error.find_one_and_delete,
                 {"w": 0},
-                results["started"][0].command["writeConcern"],
+                listener.started_events[0].command["writeConcern"],
             )
-            results.clear()
+            listener.reset()
 
         c_default.find_one_and_update({"_id": 1}, {"$set": {"foo": "bar"}})
-        self.assertNotIn("writeConcern", results["started"][0].command)
-        results.clear()
+        self.assertNotIn("writeConcern", listener.started_events[0].command)
+        listener.reset()
 
         c_default.find_one_and_replace({"_id": 1}, {"foo": "bar"})
-        self.assertNotIn("writeConcern", results["started"][0].command)
-        results.clear()
+        self.assertNotIn("writeConcern", listener.started_events[0].command)
+        listener.reset()
 
         c_default.find_one_and_delete({"_id": 1})
-        self.assertNotIn("writeConcern", results["started"][0].command)
-        results.clear()
+        self.assertNotIn("writeConcern", listener.started_events[0].command)
+        listener.reset()
 
     def test_find_with_nested(self):
         c = self.db.test
@@ -2078,7 +2107,7 @@ class TestCollection(IntegrationTest):
         c.drop()
         c.insert_one({"r": re.compile(".*")})
 
-        self.assertTrue(isinstance(c.find_one()["r"], Regex))
+        self.assertTrue(isinstance(c.find_one()["r"], Regex))  # type: ignore
         for doc in c.find():
             self.assertTrue(isinstance(doc["r"], Regex))
 
@@ -2103,6 +2132,26 @@ class TestCollection(IntegrationTest):
         with self.assertRaises(NotImplementedError):
             bool(Collection(self.db, "test"))
 
+    @client_context.require_version_min(5, 0, 0)
+    def test_helpers_with_let(self):
+        c = self.db.test
+        helpers = [
+            (c.delete_many, ({}, {})),
+            (c.delete_one, ({}, {})),
+            (c.find, ({})),
+            (c.update_many, ({}, {"$inc": {"x": 3}})),
+            (c.update_one, ({}, {"$inc": {"x": 3}})),
+            (c.find_one_and_delete, ({}, {})),
+            (c.find_one_and_replace, ({}, {})),
+            (c.aggregate, ([],)),
+        ]
+        for let in [10, "str", [], False]:
+            for helper, args in helpers:
+                with self.assertRaisesRegex(TypeError, "let must be an instance of dict"):
+                    helper(*args, let=let)  # type: ignore
+        for helper, args in helpers:
+            helper(*args, let={})  # type: ignore
+
 
 if __name__ == "__main__":
     unittest.main()
