@@ -20,6 +20,7 @@ import tempfile
 from collections import OrderedDict
 from decimal import Decimal
 from random import random
+from typing import Any, Tuple, Type, no_type_check
 
 sys.path[0:0] = [""]
 
@@ -80,7 +81,7 @@ class DecimalCodec(DecimalDecoder, DecimalEncoder):
 DECIMAL_CODECOPTS = CodecOptions(type_registry=TypeRegistry([DecimalCodec()]))
 
 
-class UndecipherableInt64Type(object):
+class UndecipherableInt64Type:
     def __init__(self, value):
         self.value = value
 
@@ -145,7 +146,8 @@ def type_obfuscating_decoder_factory(rt_type):
     return ResumeTokenToNanDecoder
 
 
-class CustomBSONTypeTests(object):
+class CustomBSONTypeTests:
+    @no_type_check
     def roundtrip(self, doc):
         bsonbytes = encode(doc, codec_options=self.codecopts)
         rt_document = decode(bsonbytes, codec_options=self.codecopts)
@@ -158,23 +160,26 @@ class CustomBSONTypeTests(object):
         self.roundtrip({"average": [[Decimal("56.47")]]})
         self.roundtrip({"average": [{"b": Decimal("56.47")}]})
 
+    @no_type_check
     def test_decode_all(self):
         documents = []
         for dec in range(3):
-            documents.append({"average": Decimal("56.4%s" % (dec,))})
+            documents.append({"average": Decimal(f"56.4{dec}")})
 
-        bsonstream = bytes()
+        bsonstream = b""
         for doc in documents:
             bsonstream += encode(doc, codec_options=self.codecopts)
 
         self.assertEqual(decode_all(bsonstream, self.codecopts), documents)
 
+    @no_type_check
     def test__bson_to_dict(self):
         document = {"average": Decimal("56.47")}
         rawbytes = encode(document, codec_options=self.codecopts)
         decoded_document = _bson_to_dict(rawbytes, self.codecopts)
         self.assertEqual(document, decoded_document)
 
+    @no_type_check
     def test__dict_to_bson(self):
         document = {"average": Decimal("56.47")}
         rawbytes = encode(document, codec_options=self.codecopts)
@@ -190,11 +195,13 @@ class CustomBSONTypeTests(object):
             bsonstream += encode(doc)
         return edocs, bsonstream
 
+    @no_type_check
     def test_decode_iter(self):
         expected, bson_data = self._generate_multidocument_bson_stream()
         for expected_doc, decoded_doc in zip(expected, decode_iter(bson_data, self.codecopts)):
             self.assertEqual(expected_doc, decoded_doc)
 
+    @no_type_check
     def test_decode_file_iter(self):
         expected, bson_data = self._generate_multidocument_bson_stream()
         fileobj = tempfile.TemporaryFile()
@@ -280,7 +287,7 @@ class TestBSONTypeEnDeCodecs(unittest.TestCase):
             else:
                 codec()
 
-        class MyType(object):
+        class MyType:
             pass
 
         run_test(
@@ -332,13 +339,22 @@ class TestBSONTypeEnDeCodecs(unittest.TestCase):
 
 
 class TestBSONCustomTypeEncoderAndFallbackEncoderTandem(unittest.TestCase):
+
+    TypeA: Any
+    TypeB: Any
+    fallback_encoder_A2B: Any
+    fallback_encoder_A2BSON: Any
+    B2BSON: Type[TypeEncoder]
+    B2A: Type[TypeEncoder]
+    A2B: Type[TypeEncoder]
+
     @classmethod
     def setUpClass(cls):
-        class TypeA(object):
+        class TypeA:
             def __init__(self, x):
                 self.value = x
 
-        class TypeB(object):
+        class TypeB:
             def __init__(self, x):
                 self.value = x
 
@@ -420,14 +436,18 @@ class TestBSONCustomTypeEncoderAndFallbackEncoderTandem(unittest.TestCase):
 
 
 class TestTypeRegistry(unittest.TestCase):
+    types: Tuple[object, object]
+    codecs: Tuple[Type[TypeCodec], Type[TypeCodec]]
+    fallback_encoder: Any
+
     @classmethod
     def setUpClass(cls):
-        class MyIntType(object):
+        class MyIntType:
             def __init__(self, x):
                 assert isinstance(x, int)
                 self.x = x
 
-        class MyStrType(object):
+        class MyStrType:
             def __init__(self, x):
                 assert isinstance(x, str)
                 self.x = x
@@ -513,7 +533,7 @@ class TestTypeRegistry(unittest.TestCase):
             def transform_bson(self, value):
                 return self.types[0](value)
 
-        codec_instances = [MyIntDecoder(), MyIntEncoder()]
+        codec_instances: list = [MyIntDecoder(), MyIntEncoder()]
         type_registry = TypeRegistry(codec_instances)
 
         self.assertEqual(
@@ -521,31 +541,30 @@ class TestTypeRegistry(unittest.TestCase):
             {MyIntEncoder.python_type: codec_instances[1].transform_python},
         )
         self.assertEqual(
-            type_registry._decoder_map, {MyIntDecoder.bson_type: codec_instances[0].transform_bson}
+            type_registry._decoder_map,
+            {MyIntDecoder.bson_type: codec_instances[0].transform_bson},
         )
 
     def test_initialize_fail(self):
-        err_msg = (
-            "Expected an instance of TypeEncoder, TypeDecoder, " "or TypeCodec, got .* instead"
-        )
+        err_msg = "Expected an instance of TypeEncoder, TypeDecoder, or TypeCodec, got .* instead"
         with self.assertRaisesRegex(TypeError, err_msg):
-            TypeRegistry(self.codecs)
+            TypeRegistry(self.codecs)  # type: ignore[arg-type]
 
         with self.assertRaisesRegex(TypeError, err_msg):
             TypeRegistry([type("AnyType", (object,), {})()])
 
-        err_msg = "fallback_encoder %r is not a callable" % (True,)
+        err_msg = f"fallback_encoder {True!r} is not a callable"
         with self.assertRaisesRegex(TypeError, err_msg):
-            TypeRegistry([], True)
+            TypeRegistry([], True)  # type: ignore[arg-type]
 
-        err_msg = "fallback_encoder %r is not a callable" % ("hello",)
+        err_msg = "fallback_encoder {!r} is not a callable".format("hello")
         with self.assertRaisesRegex(TypeError, err_msg):
-            TypeRegistry(fallback_encoder="hello")
+            TypeRegistry(fallback_encoder="hello")  # type: ignore[arg-type]
 
     def test_type_registry_repr(self):
         codec_instances = [codec() for codec in self.codecs]
         type_registry = TypeRegistry(codec_instances)
-        r = "TypeRegistry(type_codecs=%r, fallback_encoder=%r)" % (codec_instances, None)
+        r = f"TypeRegistry(type_codecs={codec_instances!r}, fallback_encoder={None!r})"
         self.assertEqual(r, repr(type_registry))
 
     def test_type_registry_eq(self):
@@ -580,7 +599,7 @@ class TestTypeRegistry(unittest.TestCase):
                 ]:
                     continue
 
-                class MyType(pytype):
+                class MyType(pytype):  # type: ignore
                     pass
 
                 attrs.update({"python_type": MyType, "transform_python": lambda x: x})
@@ -657,7 +676,7 @@ class TestCollectionWCustomType(IntegrationTest):
         )
         test = db.get_collection("test", codec_options=UNINT_DECODER_CODECOPTS)
 
-        pipeline = [
+        pipeline: list = [
             {"$match": {"status": "complete"}},
             {"$group": {"_id": "$status", "total_qty": {"$sum": "$qty"}}},
         ]
@@ -758,15 +777,18 @@ class TestGridFileCustomType(IntegrationTest):
             self.assertRaises(AttributeError, setattr, two, attr, 5)
 
 
-class ChangeStreamsWCustomTypesTestMixin(object):
+class ChangeStreamsWCustomTypesTestMixin:
+    @no_type_check
     def change_stream(self, *args, **kwargs):
         return self.watched_target.watch(*args, **kwargs)
 
+    @no_type_check
     def insert_and_check(self, change_stream, insert_doc, expected_doc):
         self.input_target.insert_one(insert_doc)
         change = next(change_stream)
         self.assertEqual(change["fullDocument"], expected_doc)
 
+    @no_type_check
     def kill_change_stream_cursor(self, change_stream):
         # Cause a cursor not found error on the next getMore.
         cursor = change_stream._cursor
@@ -774,6 +796,7 @@ class ChangeStreamsWCustomTypesTestMixin(object):
         client = self.input_target.database.client
         client._close_cursor_now(cursor.cursor_id, address)
 
+    @no_type_check
     def test_simple(self):
         codecopts = CodecOptions(
             type_registry=TypeRegistry([UndecipherableIntEncoder(), UppercaseTextDecoder()])
@@ -799,6 +822,7 @@ class ChangeStreamsWCustomTypesTestMixin(object):
         self.kill_change_stream_cursor(change_stream)
         self.insert_and_check(change_stream, input_docs[2], expected_docs[2])
 
+    @no_type_check
     def test_custom_type_in_pipeline(self):
         codecopts = CodecOptions(
             type_registry=TypeRegistry([UndecipherableIntEncoder(), UppercaseTextDecoder()])
@@ -822,6 +846,7 @@ class ChangeStreamsWCustomTypesTestMixin(object):
         self.kill_change_stream_cursor(change_stream)
         self.insert_and_check(change_stream, input_docs[2], expected_docs[1])
 
+    @no_type_check
     def test_break_resume_token(self):
         # Get one document from a change stream to determine resumeToken type.
         self.create_targets()
@@ -847,6 +872,7 @@ class ChangeStreamsWCustomTypesTestMixin(object):
         self.kill_change_stream_cursor(change_stream)
         self.insert_and_check(change_stream, docs[2], docs[2])
 
+    @no_type_check
     def test_document_class(self):
         def run_test(doc_cls):
             codecopts = CodecOptions(
@@ -871,10 +897,9 @@ class ChangeStreamsWCustomTypesTestMixin(object):
 
 class TestCollectionChangeStreamsWCustomTypes(IntegrationTest, ChangeStreamsWCustomTypesTestMixin):
     @classmethod
-    @client_context.require_no_mmap
-    @client_context.require_no_standalone
+    @client_context.require_change_streams
     def setUpClass(cls):
-        super(TestCollectionChangeStreamsWCustomTypes, cls).setUpClass()
+        super().setUpClass()
         cls.db.test.delete_many({})
 
     def tearDown(self):
@@ -891,10 +916,9 @@ class TestCollectionChangeStreamsWCustomTypes(IntegrationTest, ChangeStreamsWCus
 class TestDatabaseChangeStreamsWCustomTypes(IntegrationTest, ChangeStreamsWCustomTypesTestMixin):
     @classmethod
     @client_context.require_version_min(4, 0, 0)
-    @client_context.require_no_mmap
-    @client_context.require_no_standalone
+    @client_context.require_change_streams
     def setUpClass(cls):
-        super(TestDatabaseChangeStreamsWCustomTypes, cls).setUpClass()
+        super().setUpClass()
         cls.db.test.delete_many({})
 
     def tearDown(self):
@@ -911,10 +935,9 @@ class TestDatabaseChangeStreamsWCustomTypes(IntegrationTest, ChangeStreamsWCusto
 class TestClusterChangeStreamsWCustomTypes(IntegrationTest, ChangeStreamsWCustomTypesTestMixin):
     @classmethod
     @client_context.require_version_min(4, 0, 0)
-    @client_context.require_no_mmap
-    @client_context.require_no_standalone
+    @client_context.require_change_streams
     def setUpClass(cls):
-        super(TestClusterChangeStreamsWCustomTypes, cls).setUpClass()
+        super().setUpClass()
         cls.db.test.delete_many({})
 
     def tearDown(self):
