@@ -1,4 +1,3 @@
-# -*- coding: utf-8 -*-
 #
 # Copyright 2009-present MongoDB, Inc.
 #
@@ -14,8 +13,7 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
-"""Tests for the gridfs package.
-"""
+"""Tests for the gridfs package."""
 
 import datetime
 import sys
@@ -47,7 +45,7 @@ class JustWrite(threading.Thread):
         threading.Thread.__init__(self)
         self.fs = fs
         self.n = n
-        self.setDaemon(True)
+        self.daemon = True
 
     def run(self):
         for _ in range(self.n):
@@ -62,7 +60,7 @@ class JustRead(threading.Thread):
         self.fs = fs
         self.n = n
         self.results = results
-        self.setDaemon(True)
+        self.daemon = True
 
     def run(self):
         for _ in range(self.n):
@@ -90,7 +88,7 @@ class TestGridfs(IntegrationTest):
 
     @classmethod
     def setUpClass(cls):
-        super(TestGridfs, cls).setUpClass()
+        super().setUpClass()
         cls.fs = gridfs.GridFS(cls.db)
         cls.alt = gridfs.GridFS(cls.db, "alt")
 
@@ -141,7 +139,7 @@ class TestGridfs(IntegrationTest):
         self.fs.put(b"foo", filename="test")
         self.fs.put(b"", filename="hello world")
 
-        self.assertEqual(set(["mike", "test", "hello world"]), set(self.fs.list()))
+        self.assertEqual({"mike", "test", "hello world"}, set(self.fs.list()))
 
     def test_empty_file(self):
         oid = self.fs.put(b"")
@@ -210,7 +208,7 @@ class TestGridfs(IntegrationTest):
         self.alt.put(b"foo", filename="test")
         self.alt.put(b"", filename="hello world")
 
-        self.assertEqual(set(["mike", "test", "hello world"]), set(self.alt.list()))
+        self.assertEqual({"mike", "test", "hello world"}, set(self.alt.list()))
 
     def test_threaded_reads(self):
         self.fs.put(b"hello", _id="test")
@@ -394,7 +392,7 @@ class TestGridfs(IntegrationTest):
         f = self.fs.get_last_version(filename="empty")
 
         def iterate_file(grid_file):
-            for chunk in grid_file:
+            for _chunk in grid_file:
                 pass
             return True
 
@@ -496,7 +494,7 @@ class TestGridfsReplicaSet(IntegrationTest):
     @classmethod
     @client_context.require_secondaries_count(1)
     def setUpClass(cls):
-        super(TestGridfsReplicaSet, cls).setUpClass()
+        super().setUpClass()
 
     @classmethod
     def tearDownClass(cls):
@@ -540,7 +538,7 @@ class TestGridfsReplicaSet(IntegrationTest):
 
         # Connects, doesn't create index.
         self.assertRaises(NoFile, fs.get_last_version)
-        self.assertRaises(NotPrimaryError, fs.put, "data")
+        self.assertRaises(NotPrimaryError, fs.put, "data", encoding="utf-8")
 
 
 if __name__ == "__main__":
