@@ -1,4 +1,3 @@
-# -*- coding: utf-8 -*-
 #
 # Copyright 2015-present MongoDB, Inc.
 #
@@ -14,33 +13,24 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
-"""Tests for the gridfs package.
-"""
+"""Tests for the gridfs package."""
 import datetime
 import itertools
 import threading
 import time
+from io import BytesIO
 from test import IntegrationTest, client_context, unittest
-from test.test_replica_set_client import TestReplicaSetClientBase
-from test.utils import (
-    ignore_deprecations,
-    joinall,
-    one,
-    rs_client,
-    rs_or_single_client,
-    single_client,
-)
+from test.utils import joinall, one, rs_client, rs_or_single_client, single_client
 
 import gridfs
 from bson.binary import Binary
 from bson.int64 import Int64
 from bson.objectid import ObjectId
-from bson.py3compat import StringIO, string_type
 from bson.son import SON
 from gridfs.errors import CorruptGridFile, NoFile
 from pymongo.errors import (
     ConfigurationError,
-    ConnectionFailure,
+    NotPrimaryError,
     ServerSelectionTimeoutError,
 )
 from pymongo.mongo_client import MongoClient
@@ -52,7 +42,7 @@ class JustWrite(threading.Thread):
         threading.Thread.__init__(self)
         self.gfs = gfs
         self.num = num
-        self.setDaemon(True)
+        self.daemon = True
 
     def run(self):
         for _ in range(self.num):
@@ -67,7 +57,7 @@ class JustRead(threading.Thread):
         self.gfs = gfs
         self.num = num
         self.results = results
-        self.setDaemon(True)
+        self.daemon = True
 
     def run(self):
         for _ in range(self.num):
@@ -78,17 +68,19 @@ class JustRead(threading.Thread):
 
 
 class TestGridfs(IntegrationTest):
+    fs: gridfs.GridFSBucket
+    alt: gridfs.GridFSBucket
+
     @classmethod
     def setUpClass(cls):
-        super(TestGridfs, cls).setUpClass()
+        super().setUpClass()
         cls.fs = gridfs.GridFSBucket(cls.db)
         cls.alt = gridfs.GridFSBucket(cls.db, bucket_name="alt")
 
     def setUp(self):
-        self.db.drop_collection("fs.files")
-        self.db.drop_collection("fs.chunks")
-        self.db.drop_collection("alt.files")
-        self.db.drop_collection("alt.chunks")
+        self.cleanup_colls(
+            self.db.fs.files, self.db.fs.chunks, self.db.alt.files, self.db.alt.chunks
+        )
 
     def test_basic(self):
         oid = self.fs.upload_from_stream("test_filename", b"hello world")
@@ -102,7 +94,6 @@ class TestGridfs(IntegrationTest):
         self.assertEqual(0, self.db.fs.chunks.count_documents({}))
 
     def test_multi_chunk_delete(self):
-        self.db.fs.drop()
         self.assertEqual(0, self.db.fs.files.count_documents({}))
         self.assertEqual(0, self.db.fs.chunks.count_documents({}))
         gfs = gridfs.GridFSBucket(self.db)
@@ -120,11 +111,12 @@ class TestGridfs(IntegrationTest):
         self.assertEqual(0, self.db.fs.chunks.count_documents({}))
 
         raw = self.db.fs.files.find_one()
+        assert raw is not None
         self.assertEqual(0, raw["length"])
         self.assertEqual(oid, raw["_id"])
         self.assertTrue(isinstance(raw["uploadDate"], datetime.datetime))
         self.assertEqual(255 * 1024, raw["chunkSize"])
-        self.assertTrue(isinstance(raw["md5"], string_type))
+        self.assertNotIn("md5", raw)
 
     def test_corrupt_chunk(self):
         files_id = self.fs.upload_from_stream("test_filename", b"foobar")
@@ -139,12 +131,11 @@ class TestGridfs(IntegrationTest):
             self.fs.delete(files_id)
 
     def test_upload_ensures_index(self):
-        # setUp has dropped collections.
-        names = self.db.list_collection_names()
-        self.assertFalse([name for name in names if name.startswith("fs")])
-
         chunks = self.db.fs.chunks
         files = self.db.fs.files
+        # Ensure the collections are removed.
+        chunks.drop()
+        files.drop()
         self.fs.upload_from_stream("filename", b"junk")
 
         self.assertTrue(
@@ -203,15 +194,15 @@ class TestGridfs(IntegrationTest):
         self.alt.upload_from_stream("hello world", b"")
 
         self.assertEqual(
-            set(["mike", "test", "hello world", "foo"]),
-            set(k["filename"] for k in list(self.db.alt.files.find())),
+            {"mike", "test", "hello world", "foo"},
+            {k["filename"] for k in list(self.db.alt.files.find())},
         )
 
     def test_threaded_reads(self):
         self.fs.upload_from_stream("test", b"hello")
 
         threads = []
-        results = []
+        results: list = []
         for i in range(10):
             threads.append(JustRead(self.fs, 10, results))
             threads[i].start()
@@ -272,14 +263,14 @@ class TestGridfs(IntegrationTest):
         self.assertRaises(NoFile, self.fs.open_download_stream_by_name, "test", revision=-4)
 
     def test_upload_from_stream(self):
-        oid = self.fs.upload_from_stream("test_file", StringIO(b"hello world"), chunk_size_bytes=1)
+        oid = self.fs.upload_from_stream("test_file", BytesIO(b"hello world"), chunk_size_bytes=1)
         self.assertEqual(11, self.db.fs.chunks.count_documents({}))
         self.assertEqual(b"hello world", self.fs.open_download_stream(oid).read())
 
     def test_upload_from_stream_with_id(self):
         oid = ObjectId()
         self.fs.upload_from_stream_with_id(
-            oid, "test_file_custom_id", StringIO(b"custom id"), chunk_size_bytes=1
+            oid, "test_file_custom_id", BytesIO(b"custom id"), chunk_size_bytes=1
         )
         self.assertEqual(b"custom id", self.fs.open_download_stream(oid).read())
 
@@ -300,6 +291,7 @@ class TestGridfs(IntegrationTest):
         # Test fix that guards against PHP-237
         self.fs.upload_from_stream("empty", b"")
         doc = self.db.fs.files.find_one({"filename": "empty"})
+        assert doc is not None
         doc.pop("length")
         self.db.fs.files.replace_one({"_id": doc["_id"]}, doc)
         fstr = self.fs.open_download_stream_by_name("empty")
@@ -322,7 +314,6 @@ class TestGridfs(IntegrationTest):
             ServerSelectionTimeoutError, gfs.upload_from_stream, "test", b""
         )  # Still no connection.
 
-    @ignore_deprecations
     def test_gridfs_find(self):
         self.fs.upload_from_stream("two", b"test2")
         time.sleep(0.01)
@@ -331,8 +322,9 @@ class TestGridfs(IntegrationTest):
         self.fs.upload_from_stream("one", b"test1")
         time.sleep(0.01)
         self.fs.upload_from_stream("two", b"test2++")
-        self.assertEqual(3, self.fs.find({"filename": "two"}).count())
-        self.assertEqual(4, self.fs.find({}).count())
+        files = self.db.fs.files
+        self.assertEqual(3, files.count_documents({"filename": "two"}))
+        self.assertEqual(4, files.count_documents({}))
         cursor = self.fs.find(
             {}, no_cursor_timeout=False, sort=[("uploadDate", -1)], skip=1, limit=2
         )
@@ -380,11 +372,11 @@ class TestGridfs(IntegrationTest):
         self.assertEqual(0, self.db.fs.chunks.count_documents({"files_id": gin._id}))
 
     def test_download_to_stream(self):
-        file1 = StringIO(b"hello world")
+        file1 = BytesIO(b"hello world")
         # Test with one chunk.
         oid = self.fs.upload_from_stream("one_chunk", file1)
         self.assertEqual(1, self.db.fs.chunks.count_documents({}))
-        file2 = StringIO()
+        file2 = BytesIO()
         self.fs.download_to_stream(oid, file2)
         file1.seek(0)
         file2.seek(0)
@@ -396,18 +388,18 @@ class TestGridfs(IntegrationTest):
         file1.seek(0)
         oid = self.fs.upload_from_stream("many_chunks", file1, chunk_size_bytes=1)
         self.assertEqual(11, self.db.fs.chunks.count_documents({}))
-        file2 = StringIO()
+        file2 = BytesIO()
         self.fs.download_to_stream(oid, file2)
         file1.seek(0)
         file2.seek(0)
         self.assertEqual(file1.read(), file2.read())
 
     def test_download_to_stream_by_name(self):
-        file1 = StringIO(b"hello world")
+        file1 = BytesIO(b"hello world")
         # Test with one chunk.
-        oid = self.fs.upload_from_stream("one_chunk", file1)
+        _ = self.fs.upload_from_stream("one_chunk", file1)
         self.assertEqual(1, self.db.fs.chunks.count_documents({}))
-        file2 = StringIO()
+        file2 = BytesIO()
         self.fs.download_to_stream_by_name("one_chunk", file2)
         file1.seek(0)
         file2.seek(0)
@@ -420,63 +412,42 @@ class TestGridfs(IntegrationTest):
         self.fs.upload_from_stream("many_chunks", file1, chunk_size_bytes=1)
         self.assertEqual(11, self.db.fs.chunks.count_documents({}))
 
-        file2 = StringIO()
+        file2 = BytesIO()
         self.fs.download_to_stream_by_name("many_chunks", file2)
         file1.seek(0)
         file2.seek(0)
         self.assertEqual(file1.read(), file2.read())
 
     def test_md5(self):
-        gin = self.fs.open_upload_stream("has md5")
-        gin.write(b"includes md5 sum")
-        gin.close()
-        self.assertIsNotNone(gin.md5)
-        md5sum = gin.md5
-
-        gout = self.fs.open_download_stream(gin._id)
-        self.assertIsNotNone(gout.md5)
-        self.assertEqual(md5sum, gout.md5)
-
-        gin = self.fs.open_upload_stream_with_id(ObjectId(), "also has md5")
-        gin.write(b"also includes md5 sum")
-        gin.close()
-        self.assertIsNotNone(gin.md5)
-        md5sum = gin.md5
-
-        gout = self.fs.open_download_stream(gin._id)
-        self.assertIsNotNone(gout.md5)
-        self.assertEqual(md5sum, gout.md5)
-
-        fs = gridfs.GridFSBucket(self.db, disable_md5=True)
-        gin = fs.open_upload_stream("no md5")
+        gin = self.fs.open_upload_stream("no md5")
         gin.write(b"no md5 sum")
         gin.close()
         self.assertIsNone(gin.md5)
 
-        gout = fs.open_download_stream(gin._id)
+        gout = self.fs.open_download_stream(gin._id)
         self.assertIsNone(gout.md5)
 
-        gin = fs.open_upload_stream_with_id(ObjectId(), "also no md5")
+        gin = self.fs.open_upload_stream_with_id(ObjectId(), "also no md5")
         gin.write(b"also no md5 sum")
         gin.close()
         self.assertIsNone(gin.md5)
 
-        gout = fs.open_download_stream(gin._id)
+        gout = self.fs.open_download_stream(gin._id)
         self.assertIsNone(gout.md5)
 
 
-class TestGridfsBucketReplicaSet(TestReplicaSetClientBase):
+class TestGridfsBucketReplicaSet(IntegrationTest):
     @classmethod
     @client_context.require_secondaries_count(1)
     def setUpClass(cls):
-        super(TestGridfsBucketReplicaSet, cls).setUpClass()
+        super().setUpClass()
 
     @classmethod
     def tearDownClass(cls):
         client_context.client.drop_database("gfsbucketreplica")
 
     def test_gridfs_replica_set(self):
-        rsc = rs_client(w=self.w, read_preference=ReadPreference.SECONDARY)
+        rsc = rs_client(w=client_context.w, read_preference=ReadPreference.SECONDARY)
 
         gfs = gridfs.GridFSBucket(rsc.gfsbucketreplica, "gfsbucketreplicatest")
         oid = gfs.upload_from_stream("test_filename", b"foo")
@@ -484,10 +455,7 @@ class TestGridfsBucketReplicaSet(TestReplicaSetClientBase):
         self.assertEqual(b"foo", content)
 
     def test_gridfs_secondary(self):
-        primary_host, primary_port = self.primary
-        primary_connection = single_client(primary_host, primary_port)
-
-        secondary_host, secondary_port = one(self.secondaries)
+        secondary_host, secondary_port = one(self.client.secondaries)
         secondary_connection = single_client(
             secondary_host, secondary_port, read_preference=ReadPreference.SECONDARY
         )
@@ -497,12 +465,12 @@ class TestGridfsBucketReplicaSet(TestReplicaSetClientBase):
         gfs = gridfs.GridFSBucket(secondary_connection.gfsbucketreplica, "gfsbucketsecondarytest")
 
         # This won't detect secondary, raises error
-        self.assertRaises(ConnectionFailure, gfs.upload_from_stream, "test_filename", b"foo")
+        self.assertRaises(NotPrimaryError, gfs.upload_from_stream, "test_filename", b"foo")
 
     def test_gridfs_secondary_lazy(self):
         # Should detect it's connected to secondary and not attempt to
         # create index.
-        secondary_host, secondary_port = one(self.secondaries)
+        secondary_host, secondary_port = one(self.client.secondaries)
         client = single_client(
             secondary_host, secondary_port, read_preference=ReadPreference.SECONDARY, connect=False
         )
@@ -512,7 +480,7 @@ class TestGridfsBucketReplicaSet(TestReplicaSetClientBase):
 
         # Connects, doesn't create index.
         self.assertRaises(NoFile, gfs.open_download_stream_by_name, "test_filename")
-        self.assertRaises(ConnectionFailure, gfs.upload_from_stream, "test_filename", b"data")
+        self.assertRaises(NotPrimaryError, gfs.upload_from_stream, "test_filename", b"data")
 
 
 if __name__ == "__main__":
