@@ -60,7 +60,7 @@ class MongoThread(threading.Thread):
     """A thread that uses a MongoClient."""
 
     def __init__(self, client):
-        super(MongoThread, self).__init__()
+        super().__init__()
         self.daemon = True  # Don't hang whole test if thread hangs.
         self.client = client
         self.db = self.client[DB]
@@ -107,7 +107,7 @@ class SocketGetter(MongoThread):
     """
 
     def __init__(self, client, pool):
-        super(SocketGetter, self).__init__(client)
+        super().__init__(client)
         self.state = "init"
         self.pool = pool
         self.sock = None
@@ -116,7 +116,7 @@ class SocketGetter(MongoThread):
         self.state = "get_socket"
 
         # Call 'pin_cursor' so we can hold the socket.
-        with self.pool.get_socket({}) as sock:
+        with self.pool.get_socket() as sock:
             sock.pin_cursor()
             self.sock = sock
 
@@ -132,7 +132,7 @@ def run_cases(client, cases):
     n_runs = 5
 
     for case in cases:
-        for i in range(n_runs):
+        for _i in range(n_runs):
             t = case(client)
             t.start()
             threads.append(t)
@@ -148,7 +148,7 @@ class _TestPoolingBase(IntegrationTest):
     """Base class for all connection-pool tests."""
 
     def setUp(self):
-        super(_TestPoolingBase, self).setUp()
+        super().setUp()
         self.c = rs_or_single_client()
         db = self.c[DB]
         db.unique.drop()
@@ -158,7 +158,7 @@ class _TestPoolingBase(IntegrationTest):
 
     def tearDown(self):
         self.c.close()
-        super(_TestPoolingBase, self).tearDown()
+        super().tearDown()
 
     def create_pool(self, pair=(client_context.host, client_context.port), *args, **kwargs):
         # Start the pool with the correct ssl options.
@@ -188,10 +188,10 @@ class TestPooling(_TestPoolingBase):
         # Test Pool's _check_closed() method doesn't close a healthy socket.
         cx_pool = self.create_pool(max_pool_size=10)
         cx_pool._check_interval_seconds = 0  # Always check.
-        with cx_pool.get_socket({}) as sock_info:
+        with cx_pool.get_socket() as sock_info:
             pass
 
-        with cx_pool.get_socket({}) as new_sock_info:
+        with cx_pool.get_socket() as new_sock_info:
             self.assertEqual(sock_info, new_sock_info)
 
         self.assertEqual(1, len(cx_pool.sockets))
@@ -200,11 +200,11 @@ class TestPooling(_TestPoolingBase):
         # get_socket() returns socket after a non-network error.
         cx_pool = self.create_pool(max_pool_size=1, wait_queue_timeout=1)
         with self.assertRaises(ZeroDivisionError):
-            with cx_pool.get_socket({}) as sock_info:
+            with cx_pool.get_socket() as sock_info:
                 1 / 0
 
         # Socket was returned, not closed.
-        with cx_pool.get_socket({}) as new_sock_info:
+        with cx_pool.get_socket() as new_sock_info:
             self.assertEqual(sock_info, new_sock_info)
 
         self.assertEqual(1, len(cx_pool.sockets))
@@ -213,7 +213,7 @@ class TestPooling(_TestPoolingBase):
         # Test that Pool removes explicitly closed socket.
         cx_pool = self.create_pool()
 
-        with cx_pool.get_socket({}) as sock_info:
+        with cx_pool.get_socket() as sock_info:
             # Use SocketInfo's API to close the socket.
             sock_info.close_socket(None)
 
@@ -225,20 +225,20 @@ class TestPooling(_TestPoolingBase):
         cx_pool = self.create_pool(max_pool_size=1, wait_queue_timeout=1)
         cx_pool._check_interval_seconds = 0  # Always check.
 
-        with cx_pool.get_socket({}) as sock_info:
+        with cx_pool.get_socket() as sock_info:
             # Simulate a closed socket without telling the SocketInfo it's
             # closed.
             sock_info.sock.close()
             self.assertTrue(sock_info.socket_closed())
 
-        with cx_pool.get_socket({}) as new_sock_info:
+        with cx_pool.get_socket() as new_sock_info:
             self.assertEqual(0, len(cx_pool.sockets))
             self.assertNotEqual(sock_info, new_sock_info)
 
         self.assertEqual(1, len(cx_pool.sockets))
 
         # Semaphore was released.
-        with cx_pool.get_socket({}):
+        with cx_pool.get_socket():
             pass
 
     def test_socket_closed(self):
@@ -282,7 +282,7 @@ class TestPooling(_TestPoolingBase):
 
     def test_return_socket_after_reset(self):
         pool = self.create_pool()
-        with pool.get_socket({}) as sock:
+        with pool.get_socket() as sock:
             self.assertEqual(pool.active_sockets, 1)
             self.assertEqual(pool.operation_count, 1)
             pool.reset()
@@ -299,7 +299,7 @@ class TestPooling(_TestPoolingBase):
         cx_pool._check_interval_seconds = 0  # Always check.
         self.addCleanup(cx_pool.close)
 
-        with cx_pool.get_socket({}) as sock_info:
+        with cx_pool.get_socket() as sock_info:
             # Simulate a closed socket without telling the SocketInfo it's
             # closed.
             sock_info.sock.close()
@@ -307,12 +307,12 @@ class TestPooling(_TestPoolingBase):
         # Swap pool's address with a bad one.
         address, cx_pool.address = cx_pool.address, ("foo.com", 1234)
         with self.assertRaises(AutoReconnect):
-            with cx_pool.get_socket({}):
+            with cx_pool.get_socket():
                 pass
 
         # Back to normal, semaphore was correctly released.
         cx_pool.address = address
-        with cx_pool.get_socket({}):
+        with cx_pool.get_socket():
             pass
 
     def test_wait_queue_timeout(self):
@@ -320,16 +320,16 @@ class TestPooling(_TestPoolingBase):
         pool = self.create_pool(max_pool_size=1, wait_queue_timeout=wait_queue_timeout)
         self.addCleanup(pool.close)
 
-        with pool.get_socket({}) as sock_info:
+        with pool.get_socket():
             start = time.time()
             with self.assertRaises(ConnectionFailure):
-                with pool.get_socket({}):
+                with pool.get_socket():
                     pass
 
         duration = time.time() - start
         self.assertTrue(
             abs(wait_queue_timeout - duration) < 1,
-            "Waited %.2f seconds for a socket, expected %f" % (duration, wait_queue_timeout),
+            f"Waited {duration:.2f} seconds for a socket, expected {wait_queue_timeout:f}",
         )
 
     def test_no_wait_queue_timeout(self):
@@ -338,7 +338,7 @@ class TestPooling(_TestPoolingBase):
         self.addCleanup(pool.close)
 
         # Reach max_size.
-        with pool.get_socket({}) as s1:
+        with pool.get_socket() as s1:
             t = SocketGetter(self.c, pool)
             t.start()
             while t.state != "get_socket":
@@ -359,7 +359,7 @@ class TestPooling(_TestPoolingBase):
         socks = []
         for _ in range(2):
             # Call 'pin_cursor' so we can hold the socket.
-            with pool.get_socket({}) as sock:
+            with pool.get_socket() as sock:
                 sock.pin_cursor()
                 socks.append(sock)
 
@@ -440,7 +440,7 @@ class TestPoolMaxSize(_TestPoolingBase):
             with lock:
                 self.n_passed += 1
 
-        for i in range(nthreads):
+        for _i in range(nthreads):
             t = threading.Thread(target=f)
             threads.append(t)
             t.start()
@@ -472,7 +472,7 @@ class TestPoolMaxSize(_TestPoolingBase):
             with lock:
                 self.n_passed += 1
 
-        for i in range(nthreads):
+        for _i in range(nthreads):
             t = threading.Thread(target=f)
             threads.append(t)
             t.start()
@@ -500,9 +500,9 @@ class TestPoolMaxSize(_TestPoolingBase):
         # First call to get_socket fails; if pool doesn't release its semaphore
         # then the second call raises "ConnectionFailure: Timed out waiting for
         # socket from pool" instead of AutoReconnect.
-        for i in range(2):
+        for _i in range(2):
             with self.assertRaises(AutoReconnect) as context:
-                with test_pool.get_socket({}):
+                with test_pool.get_socket():
                     pass
 
             # Testing for AutoReconnect instead of ConnectionFailure, above,
