@@ -32,7 +32,7 @@ from pymongo.srv_resolver import _HAVE_DNSPYTHON
 WAIT_TIME = 0.1
 
 
-class SrvPollingKnobs(object):
+class SrvPollingKnobs:
     def __init__(
         self,
         ttl_time=None,
@@ -130,7 +130,14 @@ class TestSrvPolling(unittest.TestCase):
         (WAIT_TIME * 10) seconds. Also check that the resolver is called at
         least once.
         """
-        sleep(WAIT_TIME * 10)
+
+        def predicate():
+            if set(expected_nodelist) == set(self.get_nodelist(client)):
+                return pymongo.srv_resolver._SrvResolver.get_hosts_and_min_ttl.call_count >= 1
+            return False
+
+        wait_until(predicate, "Node list equals expected nodelist", timeout=100 * WAIT_TIME)
+
         nodelist = self.get_nodelist(client)
         if set(expected_nodelist) != set(nodelist):
             msg = "Client nodelist %s changed unexpectedly (expected %s)"
