@@ -147,6 +147,10 @@ class TestURI(unittest.TestCase):
         self.assertRaises(InvalidURI, parse_uri, "http://foo@foobar.com")
         self.assertRaises(ValueError, parse_uri, "mongodb://::1", 27017)
 
+        # Extra whitespace should be visible in error message.
+        with self.assertRaisesRegex(ValueError, "'27017 '"):
+            parse_uri("mongodb://localhost:27017 ")
+
         orig: dict = {
             "nodelist": [("localhost", 27017)],
             "username": None,
@@ -504,7 +508,7 @@ class TestURI(unittest.TestCase):
     def test_special_chars(self):
         user = "user@ /9+:?~!$&'()*+,;="
         pwd = "pwd@ /9+:?~!$&'()*+,;="
-        uri = "mongodb://%s:%s@localhost" % (quote_plus(user), quote_plus(pwd))
+        uri = f"mongodb://{quote_plus(user)}:{quote_plus(pwd)}@localhost"
         res = parse_uri(uri)
         self.assertEqual(user, res["username"])
         self.assertEqual(pwd, res["password"])
