@@ -17,10 +17,17 @@
 import collections
 import unittest
 
+from pymongo.errors import ConfigurationError
 from pymongo.write_concern import WriteConcern
 
 
 class TestWriteConcern(unittest.TestCase):
+    def test_invalid(self):
+        # Can't use fsync and j options together
+        self.assertRaises(ConfigurationError, WriteConcern, j=True, fsync=True)
+        # Can't use w=0 and j options together
+        self.assertRaises(ConfigurationError, WriteConcern, w=0, j=True)
+
     def test_equality(self):
         concern = WriteConcern(j=True, wtimeout=3000)
         self.assertEqual(concern, WriteConcern(j=True, wtimeout=3000))
@@ -33,7 +40,7 @@ class TestWriteConcern(unittest.TestCase):
         self.assertTrue(concern != None)  # noqa
 
     def test_equality_compatible_type(self):
-        class _FakeWriteConcern(object):
+        class _FakeWriteConcern:
             def __init__(self, **document):
                 self.document = document
 
@@ -56,7 +63,7 @@ class TestWriteConcern(unittest.TestCase):
         self.assertNotEqual(WriteConcern(wtimeout=42), _FakeWriteConcern(wtimeout=2000))
 
     def test_equality_incompatible_type(self):
-        _fake_type = collections.namedtuple("NotAWriteConcern", ["document"])
+        _fake_type = collections.namedtuple("NotAWriteConcern", ["document"])  # type: ignore
         self.assertNotEqual(WriteConcern(j=True), _fake_type({"j": True}))
 
 
