@@ -21,36 +21,19 @@ import sys
 sys.path[0:0] = [""]
 
 from test import unittest
+from test.pymongo_mocks import DummyMonitor
 from test.utils import MockPool, parse_read_preference
 
 from bson import json_util
 from pymongo.common import HEARTBEAT_FREQUENCY, clean_node
 from pymongo.errors import AutoReconnect, ConfigurationError
-from pymongo.hello import Hello
-from pymongo.hello_compat import HelloCompat
+from pymongo.hello import Hello, HelloCompat
 from pymongo.server_description import ServerDescription
 from pymongo.server_selectors import writable_server_selector
 from pymongo.settings import TopologySettings
 from pymongo.topology import Topology
 
 
-class MockMonitor(object):
-    def __init__(self, server_description, topology, pool, topology_settings):
-        pass
-
-    def cancel_check(self):
-        pass
-
-    def open(self):
-        pass
-
-    def request_check(self):
-        pass
-
-    def close(self):
-        pass
-
-
 def get_addresses(server_list):
     seeds = []
     hosts = []
@@ -61,7 +44,7 @@ def get_addresses(server_list):
 
 
 def make_last_write_date(server):
-    epoch = datetime.datetime.utcfromtimestamp(0)
+    epoch = datetime.datetime.fromtimestamp(0, tz=datetime.timezone.utc).replace(tzinfo=None)
     millis = server.get("lastWrite", {}).get("lastWriteDate")
     if millis:
         diff = ((millis % 1000) + 1000) % 1000
@@ -80,15 +63,19 @@ def make_server_description(server, hosts):
         return ServerDescription(clean_node(server["address"]), Hello({}))
 
     hello_response = {"ok": True, "hosts": hosts}
-    if server_type != "Standalone" and server_type != "Mongos":
+    if server_type not in ("Standalone", "Mongos", "RSGhost"):
         hello_response["setName"] = "rs"
 
     if server_type == "RSPrimary":
-        hello_response[HelloCompat.LEGACY_PRIMARY] = True
+        hello_response[HelloCompat.LEGACY_CMD] = True
     elif server_type == "RSSecondary":
         hello_response["secondary"] = True
     elif server_type == "Mongos":
         hello_response["msg"] = "isdbgrid"
+    elif server_type == "RSGhost":
+        hello_response["isreplicaset"] = True
+    elif server_type == "RSArbiter":
+        hello_response["arbiterOnly"] = True
 
     hello_response["lastWrite"] = {"lastWriteDate": make_last_write_date(server)}
 
@@ -122,9 +109,11 @@ def get_topology_type_name(scenario_def):
 
 
 def get_topology_settings_dict(**kwargs):
-    settings = dict(
-        monitor_class=MockMonitor, heartbeat_frequency=HEARTBEAT_FREQUENCY, pool_class=MockPool
-    )
+    settings = {
+        "monitor_class": DummyMonitor,
+        "heartbeat_frequency": HEARTBEAT_FREQUENCY,
+        "pool_class": MockPool,
+    }
     settings.update(kwargs)
     return settings
 
@@ -141,7 +130,9 @@ def create_topology(scenario_def, **kwargs):
     topology_type = get_topology_type_name(scenario_def)
     if topology_type == "LoadBalanced":
         kwargs.setdefault("load_balanced", True)
-
+    # Force topology description to ReplicaSet
+    elif topology_type in ["ReplicaSetNoPrimary", "ReplicaSetWithPrimary"]:
+        kwargs.setdefault("replica_set_name", "rs")
     settings = get_topology_settings_dict(heartbeat_frequency=frequency, seeds=seeds, **kwargs)
 
     # "Eligible servers" is defined in the server selection spec as
@@ -155,8 +146,10 @@ def create_topology(scenario_def, **kwargs):
         server_description = make_server_description(server, hosts)
         topology.on_change(server_description)
 
-    if topology_type == "LoadBalanced":
-        assert topology.description.topology_type_name == "LoadBalanced"
+    # Assert that descriptions match
+    assert (
+        scenario_def["topology_description"]["type"] == topology.description.topology_type_name
+    ), topology.description.topology_type_name
 
     return topology
 
@@ -257,12 +250,14 @@ def create_selection_tests(test_dir):
         dirname = os.path.split(dirname[-2])[-1] + "_" + dirname[-1]
 
         for filename in filenames:
+            if os.path.splitext(filename)[1] != ".json":
+                continue
             with open(os.path.join(dirpath, filename)) as scenario_stream:
                 scenario_def = json_util.loads(scenario_stream.read())
 
             # Construct test from scenario.
             new_test = create_test(scenario_def)
-            test_name = "test_%s_%s" % (dirname, os.path.splitext(filename)[0])
+            test_name = f"test_{dirname}_{os.path.splitext(filename)[0]}"
 
             new_test.__name__ = test_name
             setattr(TestAllScenarios, new_test.__name__, new_test)
